"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: importApp.py
	
	Imports an application into RAF
"""

import sys
import Globals
import jarray

from com.ibm.rational.rafw.wsadmin.websphere.application import ApplicationDeploymentProperties
from com.ibm.rational.rafw.wsadmin.websphere.application import ApplicationOptionFactory
from com.ibm.rational.rafw.wsadmin.websphere.application import AbstractApplicationOption

LOGGER = _Logger("importApp", MessageManager.RB_WEBSPHERE_WAS)

def exportTheApp( appName ):
	AdminApp.export( appName, appFile )
#endDef

def getPostInstallConfig ( appName ):
	appHash = {}
	application = AdminConfig.getid("/Deployment:"+appName+"/" ).strip()
	if (len(application) > 0):
		deployedObject = AdminConfig.showAttribute(application, "deployedObject" )
		classloader = AdminConfig.showAttribute(deployedObject, 'classloader')
		targetMappings = AdminConfig.showAttribute(deployedObject, 'targetMappings')
		
		appHash['warClassLoaderPolicy'] = AdminConfig.showAttribute( deployedObject, "warClassLoaderPolicy" )
		appHash['mode'] = AdminConfig.showAttribute( classloader, "mode" )
	#endIf
	return appHash
#endDef

#
# Main part of program
#

rafwHome = Globals.getInstallRoot()
optDict, args = SystemUtils.getopt( sys.argv, 'appsHome:;appName:;scope:;nodename:;scopename:;mode:;script_name:;version:' )
scope = AdminHelper.buildScopeTarget(optDict)

appListing = []
apps = []
if ( optDict.has_key( "appName" ) and optDict["appName"] != "ALL" and optDict["appName"] != "" and optDict["appName"] != "${APP_NAME}" and optDict["appName"] is not None ):
	apps.append( optDict["appName"] )
else:
	if(optDict["version"] == "60"):
		LOGGER.log("CRWWA1007I")
		if (optDict['scope'] != 'cell'):
			LOGGER.log("CRWWA1006W", scope)
		#endIf
		appListing = AdminApp.list().split(newline)
	else:
		appListing = AdminApp.list(scope).split(newline)
	#endIf
	
	for app in appListing:
		if (len(app) > 0):
			apps.append( app )
		#endIf
	#endFor
#endIf


version = AdminHelper.getNodeVersion()

appmgmt = AdminControl.queryNames("*,type=AppManagement")
appmgmtJMX = AdminControl.makeObjectName(appmgmt)
session=None
hashTable=java.util.Hashtable();
signature = jarray.array(["java.lang.String", "java.util.Hashtable", "java.lang.String"],java.lang.String)

for appName in apps:
	LOGGER.log("CRWWA1003I", appName)
	appFile =  optDict["appsHome"] + "/media/" + appName + ".ear"
	appProps = optDict["appsHome"] + "/properties/" + appName + ".properties"
	exportTheApp( appName )
	# createAppProps( appName, appProps )
	postOps = getPostInstallConfig( appName )
	
	deploymentProps = ApplicationDeploymentProperties()
	deploymentProps.setAppName(appName)
	deploymentProps.setClassLoaderMode(postOps['mode'])
	deploymentProps.setWarClassLoaderPolicy(postOps['warClassLoaderPolicy'])
	
	params = jarray.array([appName, hashTable, session],java.lang.Object)
	vector=AdminControl.invoke_jmx(appmgmtJMX, "getApplicationInfo", params, signature)
	# we now have a vector of AppDeploymentTask
	iterator=vector.iterator()
	while ( iterator.hasNext()) :
	    task=iterator.next()
	    if not task.isTaskDisabled():
	        taskdata=task.getTaskData()
	        if  taskdata is None :
	        	LOGGER.debug( "No task data for task '" + task.getName() 
					+ "' is defined in the application deployment options for application '" + appName + "'");
	        else :
	        	anOption = ApplicationOptionFactory.getOptions(task.getName(), taskdata);
	        	deploymentProps.addOptions(anOption);
	        #endIf
	    #endIf
	#endWhile
	deploymentProps.setVersion(version)
	deploymentProps.processFile(optDict["appsHome"])
	
#endFor
