"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: installApp.py
	
	Installs an application into WebSphere Application Server
"""

import sys
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

installAppLogger = _Logger("installApp", MessageManager.RB_WEBSPHERE_WAS)

"""
	Sets default values for specific options.
	
	If MapModulesToServers is not defined, it will be defined as 
	 .* .* WebSphere:cell=wp61_was70_nd,node=node01,server=standalone
	 or
	 .* .* WebSphere:cell=wp61_was70_nd,cluster=portal_cluster
	 
	If MapWebModToVH is not defined, it will be defined as
	.* .* default_host
"""
def _setDefaultOptions(options, optDict):
	if (options.find("-MapModulesToServers") == -1):
		## default this to -MapModulesToServers [[.* .* cluster/server]]
		if (optDict['scope'] == "cluster"):
			server = "WebSphere:cell=" + optDict['cell'] + ",cluster=" + optDict['scopename']
			options = options + "-MapModulesToServers " + " [[ .* .* " + server + " ]] "
		elif (optDict['scope'] == "server"):
			server = "WebSphere:cell=" + optDict['cell'] + ",node=" + optDict['node'] +",server="+ optDict['scopename']
			options = options + "-MapModulesToServers " + " [[ .* .* " + server + " ]] "
		#endIf
	if (options.find("-MapWebModToVH") == -1):
		## default this to -MapWebModToVH [[.* .* default_host]]
		options = options + "-MapWebModToVH [[ .* .* default_host ]] "
	return options
#endDef
	
optDict, args = SystemUtils.getopt( sys.argv, 'appName:;appProps:;appFile:;scope:;node:;cell:;scopename:;mode:' )

scope = optDict['scope']

# parse the properties into props
props = SystemUtils.getPropertyDict( optDict['appProps'] )
if (not props.has_key("APP_NAME")):
	appName = optDict['appName']
else:
	appName =  props["APP_NAME"]		
	#print "Using APP_NAME '" + appName + "' defined in " + optDict['appProps']
	installAppLogger.log("CRWWA5046I",[appName,optDict['appProps']])
	#endIf

if (WasDeploy.checkIfAppExists(appName)):
	#print "Stopping application '" + appName + "' before updating the application."
	installAppLogger.log("CRWWA5047I",[appName])
	## stop the application 
	if (scope == "cluster"):
		WasDeploy.startStopAppOnCluster(appName, optDict['scopename'], "stop")
	elif (scope == "server"):
		WasDeploy.startStopApplication( appName, optDict['node'], optDict['scopename'], "stop")
	else:
		# scope is cell
		WasDeploy.startStopAppOnModuleMappedServers(appName, "stop")
	#endIf
#endIf
	 
warClassLoaderPolicy = None
if (props.has_key("WAR_CLASS_LOADER_POLICY")):
	warClassLoaderPolicy = props["WAR_CLASS_LOADER_POLICY"]		
#endIf 

classLoaderMode = None
if (props.has_key("CLASS_LOADER_MODE")):
	classLoaderMode = props["CLASS_LOADER_MODE"]		
#endIf 

if (not props.has_key("OPTIONS")):
	# options are not defined, so parse the config file for split options and create options string
	options = WasDeploy.createOptions(optDict['appProps'])
	options = _setDefaultOptions(options, optDict)
	#print "OPTIONS after parsing " + optDict['appProps'] + "=" + options
	installAppLogger.log("CRWWA5048I",[optDict['appProps'],options])
else:
	#print "OPTIONS is defined in " + optDict['appProps']
	installAppLogger.log("CRWWA5049I",[optDict['appProps']])
	options = props["OPTIONS"]		
#endIf 

if (scope == "cluster"):
	scopeArguments = '-cluster ' + optDict['scopename']
elif (scope == "server"):
	scopeArguments = '-node ' + optDict['node'] + ' -server ' + optDict['scopename']
else:
	## scope is cell, so options has to contain -MapModulesToServers or the modules will end up mapped to the dmgr server
	if (options.find("-MapModulesToServers") == -1):
		#print "Warning: -MapModulesToServers is not defined in the options string."
		installAppLogger.log("CRWWA5050I")
		#print "         Modules will be mapped to dmgr if this is a managed cell."
	#endIf
	scopeArguments = '-cell ' + optDict['scopename']
#endIf

WasDeploy.installApp( options, scopeArguments, optDict['appFile'], appName)

# check if we were successful
appExists = WasDeploy.checkIfAppExists(appName )
if (appExists):
	WasDeploy.postInstallConfig(appName, warClassLoaderPolicy, classLoaderMode )
else:
	raise "Error: Failed to find application '" + str(appName) + "' in Application Server configuration."
#endElse 
AdminHelper.saveAndSyncCell()

