"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: appenditionTasks.py
"""
class AppEditionTasks:
	
	def __init__(self):
		self.LOGGER = _Logger("AppEditionTasks", MessageManager.RB_WEBSPHERE_WVE)
	#endDef
		
	def activateEdition(self, props):
		state = AdminTask.getEditionState(['-appName', props['APP_NAME'], '-edition', props['EDITION']])
		if (state == 'ACTIVE'):
			print "************************************"
			print props['APP_NAME'] + " edition " + props['EDITION'] + " is already active. Aborting action."
			print "************************************"
			return 'abort'
		
		print "Activating edition " + props['EDITION'] + " for application " + props['APP_NAME']
		return AdminTask.activateEdition(['-appName', props['APP_NAME'], '-edition', props['EDITION']])
	#endDef
	
	def deactivateEdition(self, props):
		state = AdminTask.getEditionState(['-appName', props['APP_NAME'], '-edition', props['EDITION']])
		if (state == 'INACTIVE'):
			print "************************************"
			print props['APP_NAME'] + " edition " + props['EDITION'] + " is already inactive. Aborting action."
			print "************************************"
			return 'abort'
		
		print "Deactivating edition " + props['EDITION'] + " for application " + props['APP_NAME']
		return AdminTask.deactivateEdition(['-appName', props['APP_NAME'], '-edition', props['EDITION']])
	#endDef
	
	def rolloutEdition(self, props):
		print "Rolling out edition " + props['EDITION'] + " for application " + props['APP_NAME']
		return AdminTask.rolloutEdition (['-appName', props['APP_NAME'], '-edition', props['EDITION'], 
								'-params', '{rolloutStrategy ' + props['ROLLOUT_STRATEGY'] +
								'}{resetStrategy ' + props['RESET_STRATEGY'] + 
								'}{groupSize ' + props['GROUP_SIZE'] + 
								'}{drainageInterval ' + props['DRAINAGE_INTERVAL'] + '}'])
	#endDef
	
	def validateEdition(self, props):
		state = AdminTask.getEditionState(['-appName', props['APP_NAME'], '-edition', props['EDITION']])
		if (state == 'ACTIVE'):
			print "************************************"
			print props['APP_NAME'] + " edition " + props['EDITION'] + " is active. Cannot validate an active edition. Aborting action."
			print "************************************"
			return 'abort'
		
		if (state == 'VALIDATE'):
			print props['APP_NAME'] + " edition " + props['EDITION'] + " is already validated. Aborting action."
			return 'abort'
		
		print "Validating edition " + props['EDITION'] + " for application " + props['APP_NAME']
		return AdminTask.validateEdition(['-appName', props['APP_NAME'], '-edition', props['EDITION']])
	#endDef
		
	def listEditions(self, appName):
		print "Listing editions for application " + appName
		editions = AdminTask.listEditions(['-appName', appName])
		print ""
		print "************************************"
		print "Application '" + appName + "' editions: "
		for edition in editions.split('\n'):
			state = AdminTask.getEditionState(['-appName', appName, '-edition', edition])
			print "\t" + edition + "\t\t(" + state + ")"
		print "************************************"
		print ""
	#endDef
#endDef

#####################################################################################
# MAIN
#####################################################################################

print "sys.argv = %s " % sys.argv
optDict, args = SystemUtils.getopt(sys.argv, 'action:;appName:;appProps:;mode:')

mode = optDict['mode']
action = optDict['action']
appName = optDict['appName']
appPropsFile = optDict['appProps']

if appPropsFile != 1:
	props = SystemUtils.getPropertyDict(appPropsFile)

appEditionTasks = AppEditionTasks()

if (mode == MODE_EXECUTE):
	success = 0
	if action == 'activateEdition':
		success = appEditionTasks.activateEdition(props)
	elif action == 'deactivateEdition':
		success = appEditionTasks.deactivateEdition(props)
	elif action == 'rolloutEdition':
		success = appEditionTasks.rolloutEdition(props)
	elif action == 'validateEdition':
		success = appEditionTasks.validateEdition(props)
	elif action == 'listEditions':
		appEditionTasks.listEditions(appName)
	else:
		raise "Unsupported action supplied: " + action

	if (success == 'false'):
		appEditionTasks.LOGGER.error("Action failed. See log file.")
		raise "Action failed. See log file."
	elif (success == 'true'):
		AdminHelper.saveAndSyncCell()
	else:
		# we listed app editions, so we don't need to fail or sync the cell
		pass 

else:
	raise "Unsupported MODE supplied: " + mode
#endIf
