"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: installApp.py
	
	Installs an application into WebSphere Application Server
"""

import sys

optDict, args = SystemUtils.getopt( sys.argv, 'appProps:;appFile:;scope:;node:;scopename:;mode:' )

# parse the properties into props
props = SystemUtils.getPropertyDict( optDict['appProps'] )
if (not props.has_key("APP_NAME")):
	raise "APP_NAME was not defined in " + optDict['appProps']
else:
	appName =  props["APP_NAME"]		
#endIf 
if (not props.has_key("WAR_CLASS_LOADER_POLICY")):
	raise "WAR_CLASS_LOADER_POLICY was not defined in " + optDict['appProps']
else:
	warClassLoaderPolicy = props["WAR_CLASS_LOADER_POLICY"]		
#endIf 

classLoaderMode = None
if (props.has_key("CLASS_LOADER_MODE")):
	classLoaderMode = props["CLASS_LOADER_MODE"]		
#endIf 

if (not props.has_key("OPTIONS")):
	# options are not defined, so parse the config file for split options and create options string
	options = WasDeploy.createOptions(optDict['appProps'])
	print "OPTIONS after parsing " + optDict['appProps'] + "=" + options
else:
	print "OPTIONS is defined in " + optDict['appProps']
	options = props["OPTIONS"]		
#endIf 

if (optDict['scope'] == "cluster"):
	scopeArguments = '-cluster ' + optDict['scopename']
elif (optDict['scope'] == "server"):
	scopeArguments = '-node ' + optDict['node'] + ' -server ' + optDict['scopename']
else:
	## scope is cell, so options has to contain -MapModulesToServers or the modules will end up mapped to the dmgr server
	if (options.find("-MapModulesToServers") is -1):
		print "Warning: -MapModulesToServers is not defined in the options string."
		print "         Modules will be mapped to dmgr if this is a managed cell."
	#endIf
	scopeArguments = '-cell ' + optDict['scopename']
#endIf

WasDeploy.installApp( options, scopeArguments, optDict['appFile'], appName)
# check if we were successful
if (props.has_key('EDITION')):
	appName = appName + '-edition' + props['EDITION']
appExists = WasDeploy.checkIfAppExists(appName)
if (appExists):
	WasDeploy.postInstallConfig(appName, warClassLoaderPolicy, classLoaderMode )
else:
	raise "Error: Failed to install application="+appName
#endElse 
AdminHelper.saveAndSyncCell()

