"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: AdminHelper.py
	
	Contains utility methods for WebSphere Application Server
"""

from SystemUtils import *
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from com.ibm.rational.rafw.wsadmin.logging import LogUtil
from com.ibm.rational.rafw.wsadmin.performance import Performance
from com.ibm.rational.rafw.wsadmin.websphere import MessageRecorder
from XmlProperty import InvalidXmlConfigException
from ConfigComparor import ConfigComparor
from ConfigReaderException import ConfigReaderException
from java.lang import System
from java.lang import Thread
from java.lang import String
from java.util.regex import Pattern

import AdminControl
import AdminConfig
import AdminTask


############## CLASS :: _AdminHelper #####################
# This class provides helper methods for WAS			 #
# Administration										 #
#														#
##########################################################
class _AdminHelper:

	###########################################################
	## 
	###########################################################
	def __init__(self):
		self.LOGGER = _Logger("AdminHelper", MessageManager.RB_WEBSPHERE_WAS)
		## by default save every time
		self.deferSync = 0
		self.compareMessageHandler = None
		self.configSaved = 0
	#endDef
	
	"""
	Description:										
		used to build scope path for getid calls		   
														   
	Inputs:											 
	   optDict = (first value return from getopt)		  
	@return The scope in the form of "/Cell:cellname/Node:nodeName/"  
	        which can be used in AdminConfig methods
	"""
	def buildScope (self, optDict):
		Performance.start("AdminHelper.buildScope");
	
		# set the cellname
		cellname = AdminControl.getCell()
		scope = ''
		
		# get the variable map at the right level in  scope
		if (optDict['scope'] == 'cell'):
			scope = '/Cell:' + cellname + '/'
		
		elif (optDict['scope'] == 'cluster'):
			if (len(optDict['scopename']) < 1):
				raise "You didn't declare clustername and yet the scope is cluster."
			#endIf
			scope = '/Cell:' + cellname + '/ServerCluster:' + optDict['scopename'] + '/'
		
		elif (optDict['scope'] == 'node'):
			if (len(optDict['scopename']) < 1):
				raise "You didn't declare nodename and yet the scope is node."
			#endIf
			scope = '/Cell:' + cellname + '/Node:' + optDict['scopename'] + '/'
		
		elif (optDict['scope'] == 'server'):
			if (len(optDict['nodename']) < 1):
				raise "You didn't declare nodename and yet the scope is servernode. A nodename must be declared to find the server."
			if (len(optDict['scopename']) < 1):
				raise "You didn't declare servername and yet the scope is server."
				#endIf
			scope = '/Cell:' + cellname + '/Node:' + optDict['nodename'] + '/Server:' + optDict['scopename'] + '/'
		
		else:
			raise "Scope: %s is invalid. Exiting!" % (optDict['scope'])
		#endIf
		Performance.end("AdminHelper.buildScope");
		return scope
	#endDef 

	"""
	Description:										
		used to build scope path for AppDeploy and JMX calls		   
														   
	Inputs:											 
	   optDict = (first value return from getopt)		  
	@return The scope in the form of "WebSphere:cell=myCell,node=myNode,server=myServer"  
	        which can be used in AppDeploy and JMX methods
	"""
	def buildScopeTarget(self, optDict):
	
		# set the cellname
		cellname = AdminControl.getCell()
		# get the variable map at the right level in  scope
		if (optDict['scope'] == 'cell'):
			scope = 'WebSphere:cell=' + cellname
		
		elif (optDict['scope'] == 'cluster'):
			if (len(optDict['scopename']) < 1):
				raise "You didn't declare clustername and yet the scope is cluster."
			#endIf
			scope = 'WebSphere:cell=' + cellname + ',cluster=' + optDict['scopename']
		elif (optDict['scope'] == 'node'):
			if (len(optDict['scopename']) < 1):
				raise "You didn't declare nodename and yet the scope is node."
			#endIf
			scope = 'WebSphere:cell=' + cellname + ',node=' + optDict['scopename']
		elif (optDict['scope'] == 'server'):
			if (len(optDict['nodename']) < 1):
				raise "You didn't declare nodename and yet the scope is servernode. A nodename must be declared to find the server."
			if (len(optDict['scopename']) < 1):
				raise "You didn't declare servername and yet the scope is server."
				#endIf
			scope = 'WebSphere:cell=' + cellname + ',node=' + optDict['nodename'] + ',server=' + optDict['scopename']
		else:
			raise "Scope: %s is invalid. Exiting!" % (optDict['scope'])
		#endIf
		return scope
	#endDef 
	
	############ isInScope(scope, oid) ########################
	# check if oid is in scope
	###############################################################
	def isInScope(self, scope, oid):
		if (scope == "node" and SystemUtils.regexp('servers', oid) == 1):
			return 0
		elif (scope == "cell" and SystemUtils.regexp('nodes', oid) == 1):
			return 0
		elif (scope == "cell" and SystemUtils.regexp('clusters', oid) == 1):
			return 0
		elif (scope == "cell" and SystemUtils.regexp('applications', oid) == 1):
			return 0
		elif (scope == "node" and SystemUtils.regexp('applications', oid) == 1):
			return 0
		elif (scope == "server" and SystemUtils.regexp('applications', oid) == 1):
			return 0
		elif (scope == "cluster" and SystemUtils.regexp('applications', oid) == 1):
			return 0
		else:
			return 1
		#endif		
	#endDef

	"""
		Returns the lowest/deepest scope for oid.
	"""
	def getScope(self, oid):
		if (SystemUtils.regexp('servers', oid) == 1):
			return "server"
		elif (SystemUtils.regexp('nodes', oid) == 1):
			return "node"
		elif (SystemUtils.regexp('clusters', oid) == 1):
			return "cluster"
		else:
			return "cell"
		#endIf
	#endDef
	
	############ buildAttrs(params, paramList) ########################
	# get attributes for modify												  
	########################################################################
	def buildAttrs(self, params, paramList):
		attrs=[]
		for key in paramList:
			attrs.append([key, params[key]])
		#endFor	
		return attrs
	#endDef	

	########### syncNodes ################################
	# 	Description:									 #
	#		Used to Synchronize nodes					#
	#													 #
	#   Inputs:										  #
	#	nodes = (comma separated list of nodes)		  #
	#		ex: "server1,server2"						#	
	######################################################
	def syncNodes (self, nodes):
		for nodeName in nodes.split(","):
			print "checking for the existence of a NodeSync MBean on node "+nodeName
			try:
				nodeSync = AdminControl.completeObjectName("type=NodeSync,node="+nodeName+",*")
				if (len(nodeSync) == 0):
					print "Error -- NodeSync MBean not found for name "+nodeName
				#endIf 
				enabled = AdminControl.getAttribute(nodeSync, "serverStartupSyncEnabled")
				if (cmp(enabled, "false") == 0):
					print "Invoking synchronization for node " + nodeSync + \
						" because serverStartupSyncEnabled is set to false..."
					AdminControl.invoke(nodeSync, "sync")
					print "Done with synchronization."
				#endIf 
			except:
				print "--## Failed to get nodesync bean: skipping sync for this node. ##--"
				continue
			#endTry
		#endFor 
		AdminConfig.save()
	#endDef 

	########### syncCell #################################
	# 	Description:									 #
	#		Used to Syncronize nodes					 #
	######################################################
	def syncCell (self):
		nodes = []
		for node in self.listNodes():
			for server in AdminConfig.list('Server', node).split(newline):
				## server will be "" if there are no servers on this node
				if (len (server) > 0):
					type = AdminConfig.showAttribute(server, 'serverType')
					if (type != 'DEPLOYMENT_MANAGER' and type != 'WEB_SERVER'):
						nodeName = AdminConfig.showAttribute(node, "name")
						print "--## %s contains a non-deploymentManger and will be synced ##--" % (nodeName)
						nodes.append(nodeName)
						break
					#endIf
				#endIf
			#endFor
		#endFor
		if (len(nodes) > 0):
			self.syncNodes(','.join(nodes))
		else:
			print "No syncable nodes found: exiting!"
		#endIf
	#endDef

	"""
		If set to true (1) then the cell will not be synchronized every time it is saved 
		by saveAndSyncCell.  The default is false.
	"""
	def setDeferSync(self, deferSync):
		self.deferSync = deferSync
	#endDef
	
	"""
		Save the changes and sync the cell.  If there have not been any changes made, 
		this method does nothing.  If setDeferSync(1) has been called, then the changes
		are saved but the cell is not synchronized.	
	"""
	def saveAndSyncCell(self):
		## only need to save and sync if there are changes made
		if (AdminConfig.hasChanges()):
			# save config
			print "Saving Config..."
			AdminConfig.save( )
			self.configSaved = 1
			if (not self.deferSync):
				#Syncronize the Cell
				print "Synchronizing Nodes..."
				self.syncCell( )	
			#endIf
		#endIf
	#endDef

	"""
	Called when the long running wsadmin process is started.  Perform any initialization of the library here.
	"""	
	def setupWsadmin(self, mode):
		self.setDeferSync(1);
		if (mode == "compare"):
			self.LOGGER.info("Delaying compare mode result display until all comparisons are complete for all actions.")
			self.compareMessageHandler = MessageRecorder()
			ConfigComparor.addMessageListener(self.compareMessageHandler)
		#endIf
	#endDef
	
	"""
	Called when the long running wsadmin process is stopped.  Perform any required shutdown tasks here.
	"""	
	def teardownWsadmin(self, mode):
		## if the config was modified by augment or execute modes
		## sync the cell
		if ( (mode == "augment" or mode == "execute") and  self.configSaved):
			AdminHelper.syncCell()
		elif (mode == "compare" and self.compareMessageHandler is not None):
			self.LOGGER.info("All action compare mode results are as follows.")
			self.compareMessageHandler.displayRecordedMessages()
		#endIf
		LogUtil.flushLogHandlers()
		Performance.displayMetrics()
	#endDef


	"""
		Returns the was version for a node in this cell as a . separated character string.  [major.minor.sub]
		Example: 6.1.0.17
		
		The first node found in the cell has its version returned.
		
		If there are no nodes in the cell, None is returned.  Of course, this is very highly unlikely.
	"""
	def getNodeVersion(self):
		nodeList = self.listNodes();
		if (len(nodeList) > 0):
			nodeId = nodeList[0];
			nodeName = AdminConfig.showAttribute(nodeId, "name")
			productVersion = AdminTask.getNodeBaseProductVersion('[-nodeName ' + nodeName + ']')
			return productVersion
		#endIf
		return None
	#endDef
			
	############ listNodes() ##############################
	# This simply lists all of the nodes				  #
	#######################################################
	def listNodes(self):
		nodeList = AdminConfig.list("Node").split(newline)
		return nodeList
	#endDef
	
	############ listServers() ############################
	# This simply lists all of the servers				#
	#######################################################
	def listServers(self):
		serverList = AdminConfig.list("Server").split(newline)
		return serverList
	#endDef
	
	############ listServers() ############################
	# This simply lists all of the servers				#
	#######################################################
	def listServersOnNode(self, nodeName):
		cellName = AdminControl.getCell()
		nodeId = AdminConfig.getid("/Cell:" + cellName + "/Node:" + nodeName)
		serverList = AdminConfig.list("Server", nodeId).split(newline)
		return serverList
	#endDef
	
	########### checkServerStarted  ######################
	# 	Description:									 #
	#		Used to verify if a server is started		#
	#													 #
	#   Inputs:										  #
	#	nodeName = (name of node server is on)		   #
	#	serverName = (name of server to verify)		  #	
	#													 #
	#   Returns:										 #
	#	0 = not started								  #
	#	1 = started									  #
	######################################################
	def checkServerStarted (self, serverName, nodeName):

		print "Checking to see if server " + serverName + " on node " + nodeName + " has started."
		serverID = ""
		retries = -1
		while ((len(serverID) == 0)  and (retries < 3)):
			retries = retries+1
			print "Checking to see if server " + serverName + " on node " + nodeName + " has started.  Attempt number " + str(retries)
			SystemUtils.sleepDelay(1)
			try:
				_excp_ = 0
				serverID = AdminControl.completeObjectName("node="+nodeName+",name="+serverName+",type=Server,*")
				excp_ = 0 #reset (in case of nested exceptions)
			except:
				_type_, _value_, _tbck_ = sys.exc_info()
				_excp_ = 1
			#endTry
			temp = _excp_
			if (temp != 0):
				self.LOGGER.debug("checkServerStarted AdminControl exception="+temp+" serverID="+serverID)
			#endIf
			self.LOGGER.debug("checkServerStarted: serverID="+serverID+"  retries="+str(retries)+" ...")
		#endWhile
		self.LOGGER.debug("Server " + serverName + " on node " + nodeName + " is not started after attempt number " + str(retries))
		if (serverID == ""):
			print "Server " + serverName + " on node " + nodeName + " IS NOT STARTED."
			return 0
		#endIf
		state = AdminControl.getAttribute(serverID, "state")
		state = state.upper()
		if (state == "STARTED"):
			print "Server " + serverName + " on node " + nodeName + " IS STARTED."
			return 1
		else:
			print "Server " + serverName + " on node " + nodeName + " is in state '" + state + "'"
			return 0
		#endElse
	#endDef

	"""
	Used to start a server				   
														 
	Inputs:										  
	   nodeName = (name of node server is on)		   
	   serverName = (name of server to start)		   
	@return: 1 if the server started, else 0
	"""
	def startServer (self, serverName, nodeName):
		started = ""
		try:
			_excp_ = 0
			if (self.checkServerStarted(serverName, nodeName) != 1):
				print "starting server '" + serverName + "' on node '"+nodeName+"'"
				#Default wait is 120 seconds, 100000 didn't seem to do anything my guess
				#is we exceeded some max, so let's try 5 minutes
				started = AdminControl.startServer(serverName, nodeName, 300)
				self.LOGGER.debug("Start server returned " + str(started))
				if(self.checkServerStarted(serverName, nodeName) != 1):
					raise "Server start time elapsed, retrying"
				else:
					print "Server " + serverName + " was successfully started"
					return 1
			else:
				print "Server '" + serverName + "' is started."
				return 1
			_excp_ = 0 #reset (in case of nested exceptions)
		except:
			_type_, _value_, _tbck_ = sys.exc_info()
			_excp_ = 1
		#endTry
		temp = _excp_
		self.LOGGER.debug("startServer: " + serverName + " errorcode="+str(temp)+" started="+ str(started) +" ...")
		retries = 0
		while ((temp > 0)  and (retries < 5)):
			retries = retries+1
			print ("Start failed for server " + serverName 
				+ " on node " + nodeName + " exception="+ str(temp) +", retries="+str(retries)+" ...")
			try:
				_excp_ = 0
				started = AdminControl.startServer(serverName, nodeName, 300)
				_excp_ = 0 #reset (in case of nested exceptions)
			except:
				_type_, _value_, _tbck_ = sys.exc_info()
				_excp_ = 1
			#endTry
			temp = _excp_
			self.LOGGER.debug("startServer: " + serverName + " errorcode="+str(temp)+" started="+ str(started) +" ...")
		#endWhile
		if (self.checkServerStarted(serverName, nodeName) == 1):
			print "Server '" + serverName + "' is started."
			return 1
		else:
			return 0
	#endDef

	########### stopServer ###############################
	# 	Description:									 #
	#		Used to stop a server						#
	#													 #
	#   Inputs:										  #
	#	nodeName = (name of node server is on)		   #
	#	serverName = (name of server to start)		   #	
	######################################################
	def stopServer (self, serverName, nodeName):
		if (self.checkServerStarted(serverName, nodeName) == 1):
			print "stopServer: nodeName="+nodeName+" serverName="+serverName+" ..."
			AdminControl.stopServer(serverName, nodeName)
		else:
			print "stopServer: nodeName="+nodeName+" serverName="+serverName+" is already stopped..."
	#endDef
	
	########### startStopServersOnNode ###################
	# 	Description:									 #
	#		Used to start/stop servers on Node		   #
	#													 #
	#   Inputs:										  #
	#	action = [start, stop, ripplestart]			  #
	#	serverList = (name of servers to start/stop)	 #	
	######################################################
	def startStopServersOnNode(self, action, nodeName):
		serverList = self.listServersOnNode(nodeName)
		
		threads = []
		for serverName in serverList:
			if (action == "start"):
				t = AdminHelper.startServerThread(serverName, nodeName)
				t.start();
				threads.append(t)
			if ( action == "stop" ):
				AdminHelper.stopServer(serverName, nodeName)
			if ( action == "ripplestart" ):
				self.stopServer(serverName, nodeName)
				self.startServer(serverName, nodeName)
			#endIf
		#endFor
		failedServers = []
		if (len(threads) > 0):
			for t in threads:
				t.join()
				if (not t.getStatus()):
					failedServers.append(t.getServerName());
				#endIf
			#endFor
		#endIf
		if (len(failedServers) > 0):
			raise MessageManager.getExceptionMessage(MessageManager.RB_WEBSPHERE_WAS, "CRWWA0400E", ",".join(failedServers))
		#endIf
	#endDef

	"""
		Finds the ProviderId in WAS that matches the provider name
		in the supplied XML Node.
		
		xmlNode - Defines the configuration that will be created.  
		          This xml node should contain a child node of type providerType
		scope - Used to create the scope in AdminConfig.getId ex: 'cell:cellName'
		providerType - The type of Provider (JDBCProvider, JMSProvider, etc.)
		attrName - The attribute on the <providerType> xml node that is 
		           used to identify the provider in WAS (typically 'name')
		altXmlProviderType - Optional the alternate name of the provider 
		                  node in the xml document.  
		                  First node of type providerType will be searched for,
		                  If a node of type providerType cannot be found, 
		                  alternateXmlProviderType will be used.  
		                  This is typically "builtin"
		           
		Returns the id of the provider if one can be found
		Throws InvalidXmlConfigException if either of the following conditions are met.
		       A. The XML configuration is missing the required <providerType> child
		       B. The XML configuration's <providerType> child is missing the 
		          attrName attribute
		Throws ConfigReaderException if either of the following conditions are met.
		       A. The XML configuration's <providerType> object cannot be found in 
		          WAS as described by the XML node
		       B. More than one of the XML configuration's <providerType> objects were found in 
		          WAS as described by the XML node
	"""
	def findProviderId(self, xmlNode, scope, providerType, attrName, altXmlProviderType=None):
		Performance.start("AdminHelper.findProviderId");
		configType = xmlNode.getXmlNode().getNodeName()
		providerNodes = xmlNode.getFilteredChildrenArray(providerType)
		if ( len(providerNodes) == 0 or providerNodes[0] == None):
			## try for the alternate
			if (altXmlProviderType is not None):
				providerNodes = xmlNode.getFilteredChildrenArray(altXmlProviderType)
			#endIf
		#endIf
			
		if ( len(providerNodes) > 0 and providerNodes[0] != None):
			providerNode = providerNodes[0]
			if (providerNode.hasAttr(attrName)):
				providerName = providerNode.getAttrValue(attrName)
				providerIdByName = self.findProviderIdByName(scope, providerType, providerName)
				if( len(providerIdByName) == 1):
					#Hopefully the nominal case, 1 provider with this name
					print "providerIdByName is " + str(providerIdByName)
					Performance.end("AdminHelper.findProviderId");
					return providerIdByName[0]
				elif( len(providerIdByName) > 2):
					#Error, more than 2 providers with the same name
					msg = "More than 2 providers with the attribute name "+ str(attrName)+ " were found. Unable to determine correct provider"
					raise ConfigReaderException(msg)
				else:
					#We have 2 providers with the same name, probably a builtin, and one created from the builtin
					#choose the non-builtin if we can find it
					for providerId in providerIdByName:
						print "providerIdByName is " + str(providerId)
						if (providerId.find('builtin_') == -1 ):
							Performance.end("AdminHelper.findProviderId");
							return providerId
						else:
							print "excluding builtin provider " + str(providerId)
						#endIf
					#endFor
				msg = "Multiple providers were returned with attribute name " + attrName + " and all were builtin"
				raise ConfigReaderException(msg)
			else:
				msg = "Attribute '" + attrName + "' is required on xmlNodes of type " + providerType
				if (altXmlProviderType is not None):
					msg += " or " + altXmlProviderType
				#endIf
				raise InvalidXmlConfigException(msg)
			#endIf
		else:
			msg = "XML node " + providerType 
			if (altXmlProviderType is not None):
				msg += " or " + altXmlProviderType
			#endIf
			msg += " does not exist as a child of " + configType
			raise InvalidXmlConfigException(msg)
		#endIf
	#endDef
	
	"""
		Finds the ProviderId in WAS that matches the provider name supplied.
		
		scope - Used to create the scope in AdminConfig.getId ex: 'cell:cellName'
		providerType - The type of Provider (JDBCProvider, JMSProvider, etc.)
		providerName - The name of the provider to locate
		           
		Returns a list of the provider ids if one can be found
		
		Throws ConfigReaderException if either of the following conditions are met.
	    A. The search for scope + providerType:providerName/ does not result in a valid WAS config id
	    
	"""
	def findProviderIdByName(self, scope, providerType, providerName):
		parentId = AdminConfig.getid( scope + providerType + ':' + providerName + '/' )
		if (len(parentId) == 0):
			msg = "Unable to find " + providerType + " with name '" + providerName + "' in scope."
			raise ConfigReaderException(msg)
		else:
			return parentId.split(newline)
		#endIf
	#endDef
	
	"""
		Extracts websphere ids from the subId as a list if any exist.
	 
		If the supplied subId is simply an attribute value, 
		an empty list is returned
	 
		If the id starts with "(cells" then there is 1 id, and that is returned.
	
		If the id starts with "[", that is an indicator that there are multiple ids,
		the string with [ ] are split on ' (' (space) and parsed, if these
		start with "(cells" then the id is appended to the returned list

		If the id contains ") " then there is more than 1 id in a list, the string
		is parsed on ' (' (space) and parsed, if these start with "(cells" then the id is appended
		to the returned list.
	
		@param subId: The id read from WebSphere that may contain 1 or more websphere ids
		@return: A list of WebSphere ids
	"""
	def extractWASIdsFromList(self, subId):
		self.LOGGER.traceEnter(subId)
		
		if (subId == None):
			return []
		#endIf
		ids = []
		if (subId.startswith('[')):
			self.LOGGER.trace("subId starts with [, removing brackets")
			## strip the brackets
			subId = subId[1:len(subId)-1]
			## process list
			
			## Strip the quotes if they exist
			if (subId.startswith('"')):
				subId = subId[1:len(subId)]
			#endIf
			if (subId.endswith('"')):
				subId = subId[0:len(subId)-1]
			#endIf
			self._splitOnParensAndAppend(subId, ids)
		elif (re.compile("\)\"? ").search(subId)):
			self._splitOnParensAndAppend(subId, ids)
		elif (re.compile("\(cells").search(subId)):
			ids.append(subId)
		elif (re.compile("\(waspolicies").search(subId)):
			ids.append(subId)
		#endIf
		self.LOGGER.traceExit(ids)
		return ids
	#endDef
	
	def _splitOnParensAndAppend(self, input, list):
		## split on ')" "' if it exists, or could be ')" ' or could be ') '
		pattern = Pattern.compile("\)\"? \"?")
		for subsubId in pattern.split(String(input)):
			# split will remove ), so append it back on if so
			if (not subsubId.endswith(')')):
				subsubId = subsubId + ')'
			#endIf
			self.LOGGER.trace("subsubId=" + str(subsubId))
			#if (len(subsubId) > 0):
			if (re.compile("\(cells").search(subsubId)):
				list.append(subsubId)
			elif (re.compile("\(waspolicies").search(subsubId)):
				list.append(subsubId)
			#endIf
		#endFor
	#endDef	
	
	def startServerThread(self, serverName, nodeName):
		class R(Thread):
			def __init__(self, serverName, nodeName):
				self.serverName = serverName
				self.nodeName = nodeName
			#endDef
			
			def run(self): 
				self.status = AdminHelper.startServer(self.serverName, self.nodeName )
			#endDef
			
			def getServerName(self):
				return self.serverName
			#endDef
			
			def getStatus(self):
				return self.status
			#endDef
		#endClass
		return R(serverName, nodeName)
	#endDef

#endClass

## define global singletons
AdminHelper = _AdminHelper()	

