"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: ConfigComparor.py
	
	This contains methods and classes to facilitate comparing WebSphere configuration data.
"""


from com.ibm.rational.rafw.wsadmin import ObjectComparator
from JavaConversionHelper import *
from java.util import ArrayList
from com.ibm.rational.rafw.wsadmin.websphere.config import LegacyWsadminConfigHelper
from java.io import StringWriter
from com.ibm.rational.rafw.wsadmin.websphere.config.compare import Compare,\
	CompareDisplay, CompareDisplayProperties
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

############## CLASS :: ConfigComparor ###################
# Use this class to compare property values inside       #
# RAFW                                                   #
#                                                        #
##########################################################
class _ConfigComparor:
	
	def __init__(self):
		self.messageListeners = None
		self.printMessages = 1
	#endDef
	
	"""
	Compares the 2 supplied jython objects by first converting them to Java
	and comparing with .equals methods.

	@return: True if the objects contain the same data and sub elements
	"""
	def compareJyObject(self, message, jyObjOne, jyObjTwo, cmpObj=None):
		javaObjOne = JavaConversionHelper.mapToJava(jyObjOne, cmpObj)
		javaObjTwo = JavaConversionHelper.mapToJava(jyObjTwo, cmpObj)
		
		objComp = ObjectComparator(100)
		
		retVal = objComp.compare(message, javaObjOne, javaObjTwo)
		if (not retVal):
			self.addMessage(message + " Objects don't match:")
			self.addMessage("Differences are:")
			for diff in objComp.getResults():
				self.addMessage("   " + diff)
			#endFor
		#endIf
		return retVal
	#endDef

	"""
	Compares the 2 supplied jython objects by calling compareJyObject
	@param typeName: Used for display purposes 
	@param wasConfig: WebSpher config object to be compared
	@param rafwConfig: RAF config object compared with
	@param comparatorObj: An optional comparator object to be used 
						 for comparison during sorting
	@return: True if the objects contain the same data and sub elements
	"""
	def compare(self, typeName, wasConfig, rafwConfig, comparatorObj=None):
		remoteConfig = LegacyWsadminConfigHelper.buildWsadminConfigs(
				JavaConversionHelper.mapToJava(wasConfig, comparatorObj))
		localConfig = LegacyWsadminConfigHelper.buildWsadminConfigs(
				JavaConversionHelper.mapToJava(rafwConfig, comparatorObj))
			
		writer = StringWriter()
		# compare it with the xml file
		compare = Compare()
		result = compare.computeComparison(typeName, localConfig, remoteConfig)
		displayProps = CompareDisplayProperties()
		displayProps.setSummaryMessage(MessageManager.getExceptionMessage(MessageManager.RB_WEBSPHERE_WAS,
						"CRWWA1503I", result.getParent().getId()));
		display = CompareDisplay(displayProps)
		display.displayComparison(result, writer)
		self.addMessage("\n" + writer.toString())
		return not result.differencesFound()
	#endDef

	"""
	Compares the 2 lists of WsadminConfig objects
	@param typeName: Used for display purposes 
	@param remoteConfig: WebSphere config object to be compared
	@param localConfig: RAF config object compared with
	
	@return: True if the objects contain the same data and sub elements
	"""
	def compareWsadminConfig(self, typeName, remoteConfig, localConfig):
		writer = StringWriter()
		# compare it with the xml file
		compare = Compare()
		result = compare.computeComparison(typeName, localConfig, remoteConfig)
		displayProps = CompareDisplayProperties()
		displayProps.setSummaryMessage(MessageManager.getExceptionMessage(MessageManager.RB_WEBSPHERE_WAS,
						"CRWWA1503I", result.getParent().getId()));
		display = CompareDisplay(displayProps)
		display.displayComparison(result, writer)
		self.addMessage("\n" + writer.toString())
		return not result.differencesFound()
	#endDef
	
	"""
	Adds a message to the list of messages after the last recorded message.
	"""	
	def addMessage(self, msg):
		if (self.printMessages):
			print msg
		else:
			for listener in self.messageListeners:
				listener.addMessage(msg)
		#endIf
	#endDef

	"""
	Adds a listener that will record messages for later display.
	
	The messages will no longer be printed when differences are found
	and can be printed later using displayRecordedMessages() on the assigned listener.
	
	The listener must be an instance of com.ibm.rational.rafw.wsadmin.websphere.MessageRecorder
	"""	
	def addMessageListener(self, listener):
		if (self.messageListeners is None):
			self.messageListeners = ArrayList();
		self.messageListeners.add(listener)
		self.printMessages = 0
	#endDef

#endClass

## define global singletons
ConfigComparor = _ConfigComparor()

