"""
	@IBM.copyright@
	
	File name: Logger.py
	
	Logger used by Jython scripts in the framework
"""

############## CLASS :: _Logger ##########################
# A simple log configuration handler which works like    #
# the JDK Logger Configuration                           #
##########################################################

import sys

from    java.util.logging import Logger as JLogger
from    java.util.logging import Level as JLevel
from    com.ibm.rational.rafw.wsadmin.logging import ClientLogManager
from    com.ibm.rational.rafw.wsadmin.logging import MessageManager
from    com.ibm.rational.rafw.wsadmin.logging import LogUtil
from com.ibm.rational.rafw.wsadmin.performance import Performance

from types import *

class _LogConfig:
	def __init__(self):
		try:
			ClientLogManager.initializeLogger()	
		except:
			print "Unable to initialize the logger: " + str(sys.exc_info()[0])
			print "    Details:" + str(sys.exc_info()[1])
		#endTry
	#endDef
#endClass

#global singleton logging configuration
LogConfig = _LogConfig()

############## CLASS :: _Logger ##########################
# A simple logger                                        #
#                                                        #
##########################################################
class _Logger:

	CATEGORY_PREFIX = ClientLogManager.WSADMIN_LOG_NAME_PREFIX + ".jython.";
		
	def __init__(self, scriptName, resourceBundleName):
		self.scriptName = scriptName 
		try:
			categoryName = _Logger.CATEGORY_PREFIX + scriptName
			self.jlogger = JLogger.getLogger(categoryName, resourceBundleName)
			## something about wsadmin breaks the default log level initialization
			## do something similar to what we'd expect for them i guess
			self.jlogger.setLevel(ClientLogManager.getEffectiveLogLevel(categoryName))
			
			JLogger.getLogger("sun.reflect").setLevel(JLevel.SEVERE)
			#handlers & formatters should be set in the logging.properites file
		except:
			## fall back to a root logger
			self.jlogger = JLogger.getLogger("")
			print "Unable to initialize the logger: " + str(sys.exc_info()[0])
			print "    Details:" + str(sys.exc_info()[1])
		#endTry
	#endDef

	############## traceEnter  ##############################
	#   Description:                                        #
	#    Used to denote entering a method                   #
	#      Lists the method name and the parameters         #
	#                                                       #
	#   Inputs:                                             #
	#                                                       #
	#########################################################
	def traceEnter(self, args = None):
		Performance.start(self.scriptName + "." + self.getCaller())
		try:
			self.jlogger.finer(self.scriptName + " > " + self.getCaller() + ": " + self._getAsString(args))	
			#endIf
		except:
			print "Unable to write the trace data: " + str(sys.exc_info()[0])
			print "    Details:" + str(sys.exc_info()[1])
		#endTry
	#endDef
	
	def traceExit(self, retVal = None):
		Performance.end(self.scriptName + "." + self.getCaller())
		try:
			self.jlogger.finer(self.scriptName + " < " + self.getCaller() + ": " + self._getAsString(retVal))
			#endIf
		except:
			print "Unable to write the trace data: " + str(sys.exc_info()[0])
			print "    Details:" + str(sys.exc_info()[1])
		#endTry
	#endDef
	
	def trace(self, message):
		try:
			self.jlogger.finer(self.scriptName + " - " + self.getCaller() + ": " + str(message))
			#endIf
		except:
			print "Unable to write the trace data: " + str(sys.exc_info()[0])
			print "    Details:" + str(sys.exc_info()[1])
		#endTry
	#endDef
	
	def isTraceEnabled(self):
		return (self.jlogger.isLoggable(JLevel.FINER))
	#endDef
	
	def debug(self, message, args=None):
		self.jlogger.log(JLevel.FINE,message,args)
	#endDef
	
	def isDebugEnabled(self):
		return (self.jlogger.isLoggable(JLevel.FINE))
	#endDef
	
	def info(self, message, args=None):
		self.jlogger.log(JLevel.INFO,message,args)
	#endDef
	
	def log(self, message, args=None):
		try:
			severity = LogUtil.getSeverityChar(message)
			self.jlogger.log(LogUtil.mapFrameworkLevel(severity), message, args)
		except:
			print "Unable to log the data: " + str(sys.exc_info()[0])
			print "    Details:" + str(sys.exc_info()[1])
		#endTry
	#endDef

	def isInfoEnabled(self):
		return (self.jlogger.isLoggable(JLevel.INFO))
	#endDef
	
	def warn(self, message,args=None):
		self.jlogger.log(JLevel.WARNING,message,args)
		#endIf
	#endDef
	
	def isWarnEnabled(self):
		return (self.jlogger.isLoggable(JLevel.WARNING))
	#endDef
	
	def error(self, message, args=None):
		self.jlogger.log(JLevel.SEVERE,message,args)
	#endDef
		
	def isErrorEnabled(self):
		return (self.jlogger.isLoggable(JLevel.SEVERE))
	#endDef
	
	def getCaller(self):
		return sys._getframe(2).f_code.co_name
	#endDef
			
	def _getAsString(self, args):
		t = type(args)
		if t == TupleType or t == ListType:
			argsAsString = ""
			for arg in args:
				if (len(argsAsString) > 0):
					argsAsString += ", "
				#endIf
				argsAsString += str(arg)
			#endFor
			return "[" + argsAsString + "]"
		else:
			return str(args)
		#endIf
	#endDef
	
#endClass
