"""
	Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: SystemUtils.py
	
	Contains utility methods for working with the components in this system
"""

############## CLASS :: _SystemUtils #####################
# This class provides utility methods for                #
# working with the modules in this system                #
#                                                        #
##########################################################

import sys
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from    org.python.modules import re
from    java.util import Map
from    java.io import FileInputStream
from    java.io import File
from	com.ibm.rational.rafw.wsadmin.util import WsadminPropertiesUtil
import Globals

import java
newline = java.lang.System.getProperty("line.separator")


WSADMIN_PROPS_REL_PATH = "/product/conf/wsadmin.properties"
PROPERTY_UPDATE_EXISTING_CONFIG = "raf.wsadmin.augment.updateExistingConfiguration"

class _SystemUtils:

	def __init__(self):
		self.LOGGER = _Logger("SystemUtils", MessageManager.RB_WEBSPHERE_WAS)
		self.updateExistingConfig = None
	#endDef

	"""	
	Tries to implement unix style getops... options are 
	separated by a ';' and if a ':' follows then it     
	requires an argument... The args are placed into a  
	jython-dictionary (Hashtable [key,value pair]       
	"""	
	def getopt(self, args, rules):
		self.LOGGER.traceEnter([args, rules])
		if (isinstance(args, "string".__class__)):
			self.LOGGER.debug("args supplied is a string, converting to a list")
			#convert to a list by space
			args = args.split()
		#endIf
		ruleArgs = {}
		for rule in rules.split(';'):
			if (rule.endswith(':')):
				#rule = '-' + rule.replace( ':','' )
				rule = rule.replace(':', '')
				ruleArgs[rule] = 1
			else:
				#rule = "-" + rule
				ruleArgs[rule] = 0
			#endif
		#endFor
	
		# Now lets walk our args
		num=0
		while num < len(args):
			if (args[num].startswith('-')):
				opt = args[num][1:len(args[num])]
				if ruleArgs.has_key(opt):
					if (ruleArgs[opt] == 1):
						# if there are still more args
						if (num + 1 < len(args)):
							# make sure that the next argument is a value and not another expected option
							val = args[num + 1]
							if (val.startswith('-') and ruleArgs.has_key(val[1:len(val)])):
								## this is another option
								self.LOGGER.trace("An argument that requires a value was supplied without a value: " 
									+ opt + ". Next argument was: " + val)
								ruleArgs[ opt ] = None
							else:
								## next argument is a valid value
								ruleArgs[ opt ] = val
								num += 1
							#endIf
						else:
							self.LOGGER.trace("An argument that requires a value was supplied without a value: " 
								+ opt + ". There are no further arguments")
							ruleArgs[ opt ] = None
						#endIf
					else:
						ruleArgs[opt] = 1
					#endIf
				else:
					# raise NameError( 'Illegal getopt argument exception. Unknown argument: ' + args[num] )
					self.LOGGER.trace("Unknown argument supplied: " + args[num])
				#endIf
			#endIf
			num += 1
		#endFor
		self.LOGGER.traceExit([ruleArgs, args])
		return ruleArgs, args
	#endDef

	############ getPropertyDict( path ) ##################
	# Reads in the property file and returns it as a dict #
	#######################################################
	def getPropertyDict(self, path):
		thisDict = {}
		properties=WsadminPropertiesUtil();
	
		try:
			properties.load(FileInputStream(path))
			self.LOGGER.log("CRWWA0005D", str(path))
			
		except:
			print "Cannot read property file %s" % (path)
			raise Exception("Cannot read property file: " + path)
		#endTry
	
		keySet = properties.keySet()
		propKeys = []
		for key in keySet:
			propKeys.append(key)
			thisDict[key] = properties.getProperty(key)
		#endWhile
	
		for key in propKeys:
			thisDict[key] = properties.getProperty(key)
			for subkey in propKeys:
				thisDict[key] = thisDict[key].replace('${' + subkey + '}', thisDict[subkey])
			#endFor
		#endFor
		return thisDict
	#endDef

	########
	# Reads a property from the supplied dictionary or java Map
	# If the key is not in the dictionary, returns the default value supplied
	########
	def readProperty(self, properties, key, defaultValue=""):
		## if properties is a map, treat it as such, otherwise assume it is a dictionary
		if (isinstance(properties, Map)):
			if (properties.containsKey(key)):
				return properties.get(key)
			else:
				return defaultValue
			#endIf
		elif (properties.has_key(key)):
			return properties[key]
		else:
			return defaultValue
		#endIf
	#endDef
		
	############ regexp( pattern, string, flags=0 ) #########
	# used for pattern matching regular expressions         #
	#########################################################
	def regexp(self, pattern, string, flags=0):
		if(re.compile(pattern, flags).search(string)==None): return 0
		else: return 1
	#endDef

	########### sleepDelay  ##############################
	# 	Description:                                     #
	#		Used to sleep for a number of seconds        #
	#   Inputs:                                          #
	#	secs = number of seconds to sleep                #
	######################################################
	def sleepDelay (self, secs):
		import time
		tstart = time.strftime("%H:%M:%S", time.localtime(time.time()))
		#IGNORE: package require java
		try:
			_excp_ = 0
			java.lang.Thread.sleep((secs * 1000))
			_excp_ = 0 #reset (in case of nested exceptions)
		except:
			_type_, _value_, _tbck_ = sys.exc_info()
			_excp_ = 1
		#endTry
		tdone = time.strftime("%H:%M:%S", time.localtime(time.time()))
		self.LOGGER.debug("sleepDelay seconds="+str(secs)+" start="+tstart+" done="+tdone)
	#endDef
	
	"""
		Returns the path supplied made relative to RAFW_HOME
		if RAFW_HOME is not in the path, then the original path is returned.
		
		The returned relative path does not contain / at the beginning but maintains
		the form of a *nix path.  For example /opt/RAFW/foo/bar results in foo/bar returned and
		c:/RAFW/foo/bar results in foo/bar
	"""
	def makePathRelativeToInstallRoot(self, absolutePath):
		absPath = File(absolutePath).getAbsolutePath()
		absPathInstallRoot = File(Globals.getInstallRoot()).getAbsolutePath()
		beginIndex = absPath.find(absPathInstallRoot)
		if (beginIndex != -1):
			endIndex = len(absPath)
			# strip the install root and the following /
			beginIndex += len(absPathInstallRoot) + 1
			relPath = absPath[beginIndex:endIndex]
			return relPath.replace("\\", "/")
		else:
			return absolutePath
	#endDef

	"""
	Returns true (1) if update should be used in augment mode for resources which already exist.  
	
	use wsadmin.properties to turn on/off this feature for augment mode.
	"""	
	def updateOnAugment(self):
		if (self.updateExistingConfig is None):
			wsadminProps = self.getPropertyDict(Globals.getInstallRoot() + WSADMIN_PROPS_REL_PATH)
			aProp = self.readProperty(wsadminProps, PROPERTY_UPDATE_EXISTING_CONFIG, "false")
			if (aProp.lower() == "true"):
				self.updateExistingConfig = 1
			else:
				self.updateExistingConfig = 0
			#endIf
		#endIf
		return self.updateExistingConfig
	#endDef
			
#endClass

## define global singletons
SystemUtils = _SystemUtils()