'''
Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 


@author: Alden Taylor
'''

from java.util import ArrayList
from java.lang import Integer

from existingcell.Node import Node
from existingcell.Server import Server

class AbstractCellProcessor:
	'''
		Defines the interface needed for the cell definition reader.  
		Default behavior for all methods is to do nothing.
	'''

	def __init__(self, cellDefinitionReader):
		'''
		Constructor
		'''
		self.cellDefinitionReader = cellDefinitionReader
		self.isAdminAgentCell = "false"
	#endDef

	def setPlatform(self, platform):
		'''
		Sets the platform this reader is running on
		'''
		self.platform = platform.lower()
	#endDef
	
	def getVariables(self):
		'''
		Returns the name/value pairs that have been read in so far as a dictionary
		'''
		return {}
	#endDef
	
	def cellDiscovered(self, cellId):
		'''
		Called by CellDefinitionReader when a cell is discovered. 
		'''
		pass
	#endDef
	
	def nodeDiscovered(self, nodeId, isAdminAgent="false"):
		'''
		Called by CellDefinitionReader when a node is discovered. 
		'''
		pass
	#endDef
	
	def serverDiscovered(self, nodeId, serverId, isAdminAgent="false"):
		'''
		Called by CellDefinitionReader when a server is discovered. 
		'''
		pass
	#endDef
	
	def clusterDiscovered(self, clusterId):
		'''
		Called by CellDefinitionReader when a cluster is discovered. 
		'''
		pass
	#endDef
	
	def setIsAdminAgentCell(self, isAdminAgent):
		self.isAdminAgentCell = isAdminAgent
	#endDef
	
	'''
	Called by any processor that needs this logic
	'''
	def getNodeTypeFromServer(self,server):
		nodeType = None
		if (server.getServerType() == Server.ADMIN_AGENT):
			nodeType=Node.ADMIN_AGENT_NODE
		elif ((self.isAdminAgentCell == "true") and (server.getServerType() == Server.APP_SERVER)):
			nodeType=Node.ADMIN_MANAGED_NODE
		elif (server.getServerType() == Server.NODE_AGENT):
			nodeType=Node.MANAGED_NODE
		elif (server.getServerType() == Server.DMGR_SERVER):
			nodeType=Node.DMGR_NODE
		return nodeType

#endClass
		
