'''
Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 


@author: Vandana Shenoy
'''
"""
	This processor will determine the type of cell that we are processing.
	It will create an object of the type of cell that we are processing.
"""

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

from existingcell.WasCellProcessor import WasCellProcessor
from existingcell.WveCellProcessor import WveCellProcessor
from existingcell.WpsCellProcessor import WpsCellProcessor
from existingcell.WesbCellProcessor import WesbCellProcessor

import AdminTask
import AdminControl

class ProcessorFactory:
	def __init__ (self, cdr):
		self.LOGGER = _Logger("ProcessorFactory", MessageManager.RB_WEBSPHERE_WAS)
		self.cellDefinitionReader = cdr
		self.processor = None	
	#endDef
	
	'''
	This function returns the relevant processorType of the cell under consideration
	'''
	def getProcessorType(self):
		self.LOGGER.traceEnter()
		dmgrNode = AdminControl.getNode()
		productName = AdminTask.getMetadataProperty ('[-nodeName ' + dmgrNode + ' -propertyName com.ibm.websphere.wbiProductShortName]') \
					+ AdminTask.getMetadataProperty ('[-nodeName ' + dmgrNode + ' -propertyName com.ibm.websphere.wxdopProductShortName]') \
					+ AdminTask.getMetadataProperty ('[-nodeName ' + dmgrNode + ' -propertyName com.ibm.websphere.esbProductShortName]')
		# If the product Version contains the string WXDOP, it means that we have a WVE dmgr. Otherwise, we are looking at a WAS dmgr.
		# For every product extension, we need to use this code to determine the type of cell that we have encountered. This code 
		# needs to be updated for existing cell for any other product extensions as well 
		if (productName == "WXDOP"):
			#check if this is an adminagent cell running on WAS 85
			wasDeployedFeatures = AdminTask.getMetadataProperty('[-nodeName ' + dmgrNode + ' -propertyName com.ibm.websphere.deployed.features]')
			adminManaged = (wasDeployedFeatures.find('com.ibm.ws.adminagent') > -1)

			if (adminManaged):
				processor = WasCellProcessor (self.cellDefinitionReader)
				self.LOGGER.traceExit("WAS")
			else:
				processor = WveCellProcessor (self.cellDefinitionReader)
				self.LOGGER.traceExit("WVE")
			#endIf
		elif ((productName == "WPS")):
			processor = WpsCellProcessor (self.cellDefinitionReader)
			self.LOGGER.traceExit("WPS")
		elif ((productName == "WPSWESB")): #its wbiShortName + esbShortName = 'WPSWESB'
			processor = WesbCellProcessor (self.cellDefinitionReader)
			self.LOGGER.traceExit("WESB")
		else:
			processor = WasCellProcessor (self.cellDefinitionReader)
			self.LOGGER.traceExit("WAS")
		#endIf
		return processor;
	#endDef
#endClass