'''
Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 


@author: Amol Gade
'''

import sys

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

try:
	if (AdminConfig is None or AdminTask is None or AdminApp is None or AdminControl is None):
		# raise a name error in case it hasn't already happened when we tried to test the above object states
		raise NameError
	else:
		sys.modules['AdminConfig'] = AdminConfig  
		sys.modules['AdminControl'] = AdminControl  
		sys.modules['AdminApp'] = AdminApp  
		sys.modules['AdminTask'] = AdminTask  
	#endIf
except NameError:
	_Logger("CellDefinitionReader", MessageManager.RB_WEBSPHERE_WAS).log("CRWWA0146E")
	raise MessageManager.getExceptionMessage(MessageManager.RB_WEBSPHERE_WAS, "CRWWA0146E", None)
#endTry

from SystemUtils import SystemUtils
from java.io import File
from com.ibm.rational.rafw.wsadmin.util import Globals
from com.ibm.rational.rafw.wsadmin.util import PropertyFileWriter
from JavaConversionHelper import JavaConversionHelper
from ConfigReader import ConfigReader
from existingcell.Constants import DR_DEFAULT_VALUE
from existingcell.Constants import NEWLINE
from existingcell.Node import Node

import AdminConfig
import AdminTask
import AdminControl

"""
	Processor that gathers IHS topology information from a runtime configuration
"""
class RemoteCellProcessor:

	def __init__(self):
		self.LOGGER = _Logger("RemoteCellProcessor", MessageManager.RB_WEBSPHERE_WAS)
		self.configReader = ConfigReader()
		self.environmentDict = {}
		self.ihsNodeNum = 0
	#endDef
	
	## Finds the configuration Id using the name 
	def findId(self, configName):
		nodes = AdminConfig.list('Node').split(NEWLINE)
		for node in nodes:
			if (node.find(configName) >= 0):
				return node
			#endIf
		#endFor
		return None
	#endDef

	def _appendEnvironmentDict(self):
		self.LOGGER.traceEnter()
		rafwHome = Globals.getInstallRoot()
		envPropsFile = rafwHome + "/work/" + "RemoteCellProcessor_work" + ".properties"
		## make sure the rafwHome+'/work/' directory exists
		workDir = File(rafwHome + "/work")
		
		if (not workDir.exists()):
			workDir.mkdirs()
		#endIf
		self.LOGGER.log("CRWWA0147I", envPropsFile)
		PropertyFileWriter().appendFile(envPropsFile, JavaConversionHelper.mapDictToJava(self.environmentDict))
		self.LOGGER.traceExit(envPropsFile)
	#endDef

	"""
		Retrieves the needed properties related to the IHS servers in a managed cell
		
		Properties are stored in self.environmentDict
	"""
	def _processWebServers(self, nodeId, serverId):
		self.LOGGER.traceEnter()	
		serverName = AdminConfig.showAttribute(serverId, 'name')
		
		for adminServerAuth in AdminConfig.list( "AdminServerAuthentication", serverId ).split( NEWLINE ):
			if (len(adminServerAuth) > 0):
				adsDict = self.configReader.show(adminServerAuth)
				if (adsDict.has_key("userid") and adsDict["userid"] == ""):
					self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_CREATE_ADMIN_AUTH"] = "false"
				else:
					self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_CREATE_ADMIN_AUTH"] = "true"
					if (adsDict.has_key("userid")):
						self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_AUTH_USERNAME"] = adsDict["userid"]
					else:
						self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_AUTH_USERNAME"] = DR_DEFAULT_VALUE
						self.LOGGER.warn("userid cannot be read for server: " + serverName + ", defaulting to:" + DR_DEFAULT_VALUE)
					#endIf
					if (adsDict.has_key("password")):
						self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_AUTH_PASSWORD"] = adsDict["password"]
					else:
						self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_AUTH_PASSWORD"] = DR_DEFAULT_VALUE
						self.LOGGER.warn("password cannot be read for server: " + serverName + ", defaulting to:" + DR_DEFAULT_VALUE)
					#endIf
				#endIf
			#endIf
		#endFor
		
		for webServer in AdminConfig.list( "WebServer", serverId ).split( NEWLINE ):
			if (len(webServer) > 0):
				adsDict = self.configReader.show(webServer)
				ihsHome = adsDict["webserverInstallRoot"]
				## fix the path on ihs home on windows
				ihsHome = ihsHome.replace("\\" , "/")
				self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_IHS_HOME"] = ihsHome
				
				for pluginProps in AdminConfig.list( "PluginProperties", webServer ).split( NEWLINE ):
					if (len(pluginProps) > 0):
						pluginDict = self.configReader.show(pluginProps)
						self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_PLUGIN_HOME"] = pluginDict["PluginInstallRoot"].replace("\\" , "/")
				#endFor
			#endIf
		#endFor
		
		node = Node(nodeId) #Node(nodeId)
		portsByServer = node.getPorts()
		ports = portsByServer[serverName]
		if (ports.has_key("WEBSERVER_ADMIN_ADDRESS")):
			self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_IHS_ADMIN_PORT"] = ports["WEBSERVER_ADMIN_ADDRESS"]
		else:
			self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_IHS_ADMIN_PORT"] = "8008"
		#endIf
		if (ports.has_key("WEBSERVER_ADDRESS")):
			self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_HTTP_PORT"] = ports["WEBSERVER_ADDRESS"]
		else:
			self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_HTTP_PORT"] = "80"
		#endIf
	
		self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_HOST_NAME"] = AdminConfig.showAttribute(nodeId, "hostName")
		
		nodeName = node.getNodeName()
		computedNodeName = "ihs_" + nodeName + "_" + serverName
		self.environmentDict["IHS_VERSION"] = node.getNodeVersion()
		self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_NODE_NAME"] = computedNodeName
		self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_NODE_TYPE"] = "IHS"
		self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_PROFILE_NAME"] = nodeName
		self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_WEB_SERVER_DEF_NAME"] = serverName
		self.environmentDict["IHS_NODE" + str(self.ihsNodeNum) + "_ALLOW_NON_ROOT_SILENT_INSTALL"] = "false"
		self.LOGGER.traceExit()
	#endDef
	
#endClass

#  MAIN
optDict, args = SystemUtils.getopt( sys.argv, 'ihsNodeNum:;nodeName:;serverId:' )
rcp = RemoteCellProcessor()
rcp.ihsNodeNum = optDict['ihsNodeNum']
rcp._processWebServers(rcp.findId(optDict['nodeName']), optDict['serverId'])
rcp._appendEnvironmentDict()
