'''
Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 


@author: Per Bergman
'''
import java.io.File


from existingcell.wps.DepEnvAnalyzer import DepEnvAnalyzer
from existingcell.wps.CustomClusterAnalyzer import CustomClusterAnalyzer
from existingcell.wps.FourClusterAnalyzer import FourClusterAnalyzer
from existingcell.wps.BPMConfigReader import BPMConfigReader
from existingcell.wps.SimpleXPath import SimpleXPath

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

from existingcell.Constants import DR_DEFAULT_VALUE
from existingcell.Constants import NEWLINE
from existingcell.BPMCellProcessor import BPMCellProcessor

import AdminConfig
import AdminTask
import AdminControl

'''
    Existing cell extractor for WPS cell.
'''
class WpsCellProcessor(BPMCellProcessor):

    def __init__(self, cellDefinitionReader):
        BPMCellProcessor.__init__(self, cellDefinitionReader)
        self.LOGGER = _Logger("WpsCellProcessor", MessageManager.RB_WEBSPHERE_BPM)
        self.LOGGER_WAS = _Logger("WpsCellProcessor.WAS", MessageManager.RB_WEBSPHERE_WAS)
        self.cell=""
    #endDef

    '''
        Get all deployment environment ids.
        If == 1: dep env cell found. NumCluster has to be 1/2/3.
        If > 1: dep env cell found, choose first one, print message.
        If dep env: Decide on topology type and assert NumCuster
        Process dep env.
        else
            Custom cluster found.
            If NumCluster is 1 or 2:
                print message and abort, i.e. this is a WAS cell
            If NumCluster is 3 or 4: gold or platinum.
            If NumCluster is > 4: platinum + WAS clusters
    '''
    def cellDiscovered(self, cellId):
        self.LOGGER_WAS.log("CRWWA0137I", self.cellType)
        self.cell = self.adjustId(cellId)
        des = self.findDEs()
        
        fileName = ''
        depenvName = ''
        isPlatinum = "false"
        if(len(des) == 1):
            depenvName = des[0]
            self.LOGGER.log("CRWBP0100I", depenvName)
            fileName = '/tmp/' + depenvName.replace(' ', '_') + '.xml'
            deFile=java.io.File(fileName)
            if(deFile.exists()):
                deFile.delete()
            AdminTask.exportDeploymentEnvDef('-filePath ' + fileName + ' -topologyName ' + depenvName)
        elif (len(des) > 1):
            self.LOGGER.log("CRWBP0101E", len(des))
            return
        else: #This is a four cluster (platinum) DE
            self.LOGGER.log("CRWBP0102I")
            isPlatinum = "true"
        #endIf
            
        # The WBI_DataSource is created at dmgr profile creation (cell scope), regardless of topology.
        configReader = BPMConfigReader(self.cell)
        res = configReader.findCellDataSource('WBI_DataSource')
        if (res == None):
            self.LOGGER.log("CRWBP0113E")
            return
        #endIf
        
        # provider type for Oracle is: "Oracle JDBC Driver (XA)"
        # provider type for DB2 is: "DB2 Universal JDBC Driver Provider"
        isDB2 = res['providerType'][0:3]=="DB2"
        
        if (isPlatinum == "false"):
            depEnvPattern = self.getTopologyPattern(fileName)
            if(depEnvPattern == "Reference"):
                p = DepEnvAnalyzer(fileName, self, self.cellType, isDB2==0, configReader)
                p.process()
            elif(depEnvPattern == "Custom"):
                p = CustomClusterAnalyzer(fileName, self, self.cellType, isDB2==0, configReader)
                p.process()
            else:
                self.LOGGER.log("CRWBP0103E")
                return
        else:
            p = FourClusterAnalyzer(self, self.cellType, isDB2==0, configReader)
            p.process()
        #endIf
          
        alias = configReader.findCellAuthAlias(res['authDataAlias'])
        variables = configReader.getCellVariables()

        if(isDB2):
            self.add("JDBC_DRIVER_PATH",  variables['DB2UNIVERSAL_JDBC_DRIVER_PATH'])
            self.add("JDBC_DRIVER_TYPE", res['driverType'])
            self.add("WBI_DB_NAME", res['databaseName'])
            self.add("WBI_DB_PORT", res['portNumber'])
            self.add("WBI_DB_SERVER", res['serverName'])
            self.add("WBI_DB_TYPE", "DB2_Universal")
            self.add("WBI_DB_VERSION", '9.1') # For SIBus generator only, 9.1 covers all supported versions.
            if (self.cellType == "WPS"):
                self.add("WPS_DB_BPC_VERSION", self.lookup("WBI_DB_VERSION")[0:1]) # 9
            self.add("WBI_DB_USER", alias['userId'])
            self.add("WBI_DB_PASSWORD", alias['password'])
            self.add("WBI_DB_COMMON_USERNAME", alias['userId'])
            self.add("WBI_DB_COMMON_PASSWORD", alias['password'])
            self.add("WBI_DB_COMMON_SCHEMA_NAME", res['currentSchema']) # Only Z/OS DB2
        else:
            self.add("JDBC_DRIVER_PATH",  variables['ORACLE_JDBC_DRIVER_PATH'])
            self.add("JDBC_DRIVER_TYPE", "Thin")
            self.add("WBI_DB_NAME", res['databaseName'])
            self.add("WBI_DB_PORT", res['portNumber'])
            self.add("WBI_DB_SERVER", res['serverName'])

            oraKind = res['datasourceHelperClassname']    # Oracle10gDataStoreHelper
            oraVersion="10g"
            oraKind = "Oracle10g"
            if(oraKind.find("Oracle11g") != -1):
                oraKind = "Oracle11g"
                oraVersion="11g"
            else:
                pass # Error

            self.add("WBI_DB_TYPE", oraKind) # "Oracle10g", "Oracle11g"
            self.add("WBI_DB_VERSION", oraVersion) # 10g, 11g - for SIBus generator only

            dbBPCVersion = self.lookup("WBI_DB_VERSION")
            if (self.cellType == "WPS"):
                self.add("WPS_DB_BPC_VERSION", dbBPCVersion[0:2]) # 10 or 11

            self.add("WBI_DB_USER", alias['userId'])
            self.add("WBI_DB_PASSWORD", alias['password'])
            self.add("WBI_DB_COMMON_USERNAME", alias['userId'])
            self.add("WBI_DB_COMMON_PASSWORD", alias['password'])
            self.add("WBI_DB_COMMON_SCHEMA_NAME", alias['userId'])
        #endif
        
        self.add("WBI_DB_OS", DR_DEFAULT_VALUE) # Used by SIB generator

        if (self.cellType == "WPS"):
            self.add("WPS_DB_BPC_DB_HOME", DR_DEFAULT_VALUE) # Any directory that exists

        #self.add(self.cellType + "_VERSION", self.lookup(self.cellType + "_VERSION"))
        self.add("PRODUCT_VERSION", self.cellType + self.lookup(self.cellType + "_VERSION"))
        self.add("WBI_DEPLOYMENT_ENVIRONMENT_NAME", depenvName) # Noop for four cluster topology
        self.add("WBI_DEPLOYMENT_NODE", "false")
        self.add("WBI_INSTALL_TYPE", "typical")
        self.add("WBI_IS_NEW_INSTALL", "installNew")
        self.LOGGER.traceExit()
    #endDef

    '''
    format: Containment Path ('/Cell:/' + cell + '/') or Configuration ID

    Configuration IDs have the following syntax:

    name(directory|filename#XML-fragment-id)
    server1(cells/was-70Cell01/nodes/was-70Node01/servers/server1|server.xml#Server_1254421834316)
    (cells/was-70Cell01|security.xml#JAASAuthData_1258718081490)

    '''
    def adjustId(self, inId):
        ret = inId
        if(inId[0] == '/'):
            ret = AdminConfig.getid(inId)
        return ret
    #endDef

    '''
    Finds the names of all deployment environments.
    '''
    def findDEs(self):
        ret = []
        des = AdminConfig.list("WBITopology").split(NEWLINE)
        if(len(des) > 0 and des[0] != ''):
            for de in des:
                depenv = AdminConfig.showAttribute(de, "name")
                ret.append(depenv)
        return ret
    #endDef

    def getTopologyPattern(self, fileName):
        parser = SimpleXPath(fileName)
        pattern = parser.findOneByTag("pattern", None, None)
        if (pattern == None):
            return "Custom"
        else:
            return parser.findAttrValue(pattern, "id")
        
    #endDef

#endClass
