'''
Created on Nov 15, 2010

@author: Per Bergman
'''

from ConfigReader import ConfigReader
import AdminConfig
import AdminControl
from OracleUtil import OracleUtil
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

from existingcell.Constants import DR_DEFAULT_VALUE
from existingcell.Constants import NEWLINE

'''
'''
class BPMConfigReader:

    def __init__(self, cell):
        self.LOGGER = _Logger("BPMConfigReader", MessageManager.RB_WEBSPHERE_BPM)
        self.cell = cell
        self.variables = {}
        self.aliases = {}
        self.dataSources = {}
        self.oraUtil = OracleUtil()
        self.rd = ConfigReader()
    #endDef

    def findCellAuthAlias(self, jaasAlias):
        ret = {}
        if(len(self.aliases) == 0):
            self.aliases = self.rd.readConfigDataUsingParentId(self.cell, 'cell', 'JAASAuthData')

        for alias in self.aliases:
            entry = alias['attrs']
            if entry['alias'] == jaasAlias:
                ret['userId'] = entry['userId']
                ret['password'] = entry['password']
                break
        return ret
    #endDef

    def findCellDataSource(self, dsName):
        if(len(self.dataSources) == 0):
            self.dataSources = self.rd.readConfigDataUsingParentId(self.cell, 'cell', 'DataSource')

        for dataSource in self.dataSources:
            entry = dataSource['attrs']
            if entry['name'] == dsName:
                return self.loadDataSource(dataSource)
        #endFor
        return None
    #endDef

    def getCellVariables(self):
        if(len(self.variables) > 0):
            return self.variables

        for varSub in AdminConfig.list('VariableSubstitutionEntry', self.cell).split(NEWLINE):
            if (len(varSub) > 0):
                varDict = self.rd.show(varSub)
                #WAS 60 doesn't return the value attribute if it's blank
                if (not varDict.has_key('value')) :
                    varDict['value'] = ""
                self.variables[varDict['symbolicName']] = varDict['value']
        return self.variables
    #endDef

    def loadDataSource(self, dataSource):
        entry = dataSource['attrs']
        self.LOGGER.log("CRWBP0114I", entry['name'])

        ret = {}
        ret['providerType'] = entry['providerType']
        ret['datasourceHelperClassname'] = entry['datasourceHelperClassname']
        ret['jndiName'] = entry['jndiName']
        ret['name'] = entry['name']
        
        if 'authDataAlias' in entry.keys():
            ret['authDataAlias'] = entry['authDataAlias']

        attr_array = dataSource['children']
        for attr in attr_array:
            if attr['attrs']['WASKey'] == 'mapping':
                newAlias = attr['attrs']['authDataAlias']
                if (newAlias != ''):
                    ret['authDataAlias'] = newAlias #inner authDataAlias takes precedence
            #endIf
            if attr['attrs']['WASKey'] == 'propertySet':
                for resource in attr['children']:
                    if ((resource['attrs'].has_key('name')) and (resource['attrs'].has_key('value'))):
                        ret[resource['attrs']['name']] = resource['attrs']['value']
            #endIf
        #endFor
        
        isOracle = ''
        if ret['providerType'][0:3]=="DB2":
            isOracle = 'false'
        else:
            isOracle = 'true'
        #endIf
        
        if(isOracle == 'true'):
            jdbcURL = ret['URL']
            ret['databaseName'], [ret['serverName'], ret['portNumber']] = self.oraUtil.splitConnection(jdbcURL)
        #endIf
        return ret
    #endDef

    def findBPMSchema(self, clusterName, configName, attrName):
        clusterId = AdminConfig.getid('/ServerCluster:' + clusterName + '/')
        obj = AdminConfig.list(configName, clusterId)
        return AdminConfig.showAttribute(obj, attrName)
    #endDef


#endClass
