'''
Licensed Materials - Property of IBM Corp. 
IBM Rational Automation Framework 
(c) Copyright IBM Corporation 2003, 2012. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 


@author: Per Bergman
'''

import org.w3c.dom.Node as Node

from DepEnvAnalyzer import DepEnvAnalyzer

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

class CustomClusterAnalyzer(DepEnvAnalyzer):

    def __init__(self, file, collector, cellType, isOracle, bpmReader):
        self.LOGGER = _Logger("CustomClusterAnalyzer", MessageManager.RB_WEBSPHERE_BPM)
        DepEnvAnalyzer.__init__(self, file, collector, cellType, isOracle, bpmReader)
        self.compMap = {"WBI_CEI":"WBI_DB_CEI_", "WBI_BPC":"WPS_DB_BPC_", "WBI_BPCEventCollector":"WPS_DB_BPC_EVENT_", "WBI_BSPACE":"WBI_DB_BSPACE_", "WBI_SCA":"WBI_DB_SCA_SYS_ME_"}
    #endDef

    def process(self):
        self.collect("WBI_DEPLOYMENT_ENVIRONMENT_TOPOLOGY", "Custom")
        self.collect("WBI_ENVIRONMENT_TYPE", "custom")
        
        roles = self.parser.findByTag("roles", None, None)
        for role in roles:
            roleType = self.parser.findAttrValue(role, "name")
            self.LOGGER.log("CRWBP0106I", roleType)
            clusterName = self.parser.findOneByTagAndNode(role, "deploymentTarget", None, Node)
            cn = self.parser.findAttrValue(clusterName, "name")
            if ((roleType == "AppTarget") or (roleType == "Application")):
                self.collect("WBI_APPLICATION_CLUSTER_NAME", cn)
            elif (roleType == "Support"):
                self.collect("WBI_SUPPORT_CLUSTER_NAME", cn)
            elif (roleType == "Messaging"):
                self.collect("WBI_MESSAGING_CLUSTER_NAME", cn)
            else:
                self.LOGGER.log("CRWBP0109I")
        #endFor
        self.processCompMap()
    #endDef
    
    def processCompMap(self):
        keys = self.compMap.keys()
        for compId in keys:
            component = self.parser.findOneByTag("components", "id", compId)
            if (component != None):
                if (compId == "WBI_CEI"):
                    self.handleComponent(compId, self.compMap.get(compId), "", 0)
                    self.handleBus(compId, "WBI_DB_CEI_ME_", "CEI_bus")
                    self.handleJMSAuth(compId, "WBI_JMS_", "CommonEventInfrastructureJMSAuthAlias")
                elif ((compId == "WBI_BPC") and (self.cellType == "WPS")):
                    self.handleComponent(compId, "WPS_DB_BPC_", None, 1)
                    self.handleBus(compId, "WPS_DB_BPC_ME_", "BPC_bus")
                    self.handleJMSAuth(compId, "WPS_JMS_BPC_", "BPC_Auth_Alias")
                elif ((compId == "WBI_BPCEventCollector") and (self.cellType == "WPS")):
                    self.handleComponent(compId, self.compMap.get(compId), None, 0)
                elif (compId == "WBI_SCA"):
                    self.handleBus(compId, self.compMap.get(compId), "SCA_system_bus")
                    self.handleBus(compId, "WBI_DB_SCA_APP_ME_", "SCA_app_bus")
                else:
                    self.handleComponent(compId, self.compMap.get(compId), None, 0)
                #endIf
            #endIf
        #endFor
    #endDef
    
#endClass    
