'''
Created on Nov 24, 2010

@author: Per Bergman
'''

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

import java
from java.util.regex import Pattern

from OracleUtil import OracleUtil
from ConfigReader import ConfigReader
from Analyzer import Analyzer

import AdminConfig

from existingcell.Constants import DR_DEFAULT_VALUE
from existingcell.Constants import NEWLINE

'''
'''
class FourClusterAnalyzer(Analyzer):

    def __init__(self, collector, cellType, isOracle, bpmReader):
        self.LOGGER = _Logger("FourClusterAnalyzer", MessageManager.RB_WEBSPHERE_BPM)
        Analyzer.__init__(self, collector, cellType, isOracle, bpmReader)
        self.numNodes = -1
        self.numServers = -1
        self.oraUtil = OracleUtil()
        
        self.bpc=Pattern.compile('jdbc/com.ibm.ws.sib/.+-BPC\.\S+\.Bus')
        self.cei=Pattern.compile('jdbc/com.ibm.ws.sib/.+-CommonEventInfrastructure_Bus')
        self.scaa=Pattern.compile('jdbc/com.ibm.ws.sib/.+-SCA\.APPLICATION\.\S+\.Bus')
        self.scas=Pattern.compile('jdbc/com.ibm.ws.sib/.+-SCA\.SYSTEM\.\S+\.Bus')

        self.messagingJNDI = {} # Map ME JNDI name to type
        self.foundBPC = int(0) # Increment when first BPC cluster found, in platinum we have two BPC clusters.
        self.clusterDS = {} # Maps (only) cluster type to list of ds
        self.clusterName = {} # Maps cluster type to cluster name
    #endDef

    '''
    Application:
    "BPEDataSourceOracle" - jdbc/BPEDB

    Web:
    "BPEDataSourceOracle" - jdbc/BPEDB

    Messaging:
    "Business Process Choreographer ME data source" - jdbc/com.ibm.ws.sib/Messaging-BPC.cell.Bus
    "CEI ME data source" - jdbc/com.ibm.ws.sib/Messaging-CommonEventInfrastructure_Bus
    "SCA Application Bus ME data source" - jdbc/com.ibm.ws.sib/Messaging-SCA.APPLICATION.cell.Bus
    "SCA System Bus ME data source" - jdbc/com.ibm.ws.sib/Messaging-SCA.SYSTEM.cell.Bus

    Support:
    "Business Space Datasource" - jdbc/bpm/BusinessSpace
    "event" jdbc/cei
    "event_catalog" - jdbc/eventcatalog

    * ASSERTIONS *
    len(jndiNames) > 0
    jndiNames are unique per scope (cluster in this case)

    We will wrongly match a Messaging engine with names like:
    'jdbc/com.ibm.ws.sib/DummyCluster-BPC.DummyCell.Bus'
    'jdbc/com.ibm.ws.sib/DummyCluster-CommonEventInfrastructure_Bus'
    'jdbc/com.ibm.ws.sib/DummyCluster-SCA.APPLICATION.DummyCell.Bus'
    'jdbc/com.ibm.ws.sib/DummyCluster-SCA.SYSTEM.DummyCell.Bus'

    * MISSING *
    'jdbc/OBSVRDB_Support' (cell level) - can't get DB2 schema name
    'jdbc/OBSVRDB_Support_WEB' (cell level) - can't get DB2 schema name

    '''
    def decideCluster(self, jndiNames):
        if 'jdbc/BPEDB' in jndiNames:
            if(self.foundBPC == 0):
                self.foundBPC = self.foundBPC + 1
                return 'Application'
            else:
                return 'Web'
            #endIf
        #endIf

        found=int(0)
        for jndi in jndiNames:
            if (self.bpc.matcher(java.lang.String(jndi)).matches() > 0):
                self.messagingJNDI[jndi] = 'BPC'
                found = found + 1
            elif (self.cei.matcher(java.lang.String(jndi)).matches() > 0):
                self.messagingJNDI[jndi] = 'CEI'
                found = found + 1
            elif (self.scaa.matcher(java.lang.String(jndi)).matches() > 0):
                self.messagingJNDI[jndi] = 'SCAAPP'
                found = found + 1
            elif (self.scas.matcher(java.lang.String(jndi)).matches() > 0):
                self.messagingJNDI[jndi] = 'SCASYS'
                found = found + 1
            #endIf
        #endFor

        if(found == 4):
            return 'Messaging'

        return 'Support' #default
    #endDef

    def process(self):
        rd = ConfigReader()

        obs = self.getBPMReader().findCellDataSource('Business Process Choreographer reporting function source')
        obsWeb = self.getBPMReader().findCellDataSource('Business Process Choreographer reporting function source Web Cluster')
        if(obs != None):
            self.handleObserver(obs, 'WPS_DB_BPC_EVENT_')
        if(obsWeb != None):
            self.handleObserver(obsWeb, 'WPS_DB_BPC_EVENT_WEB_')

        # Iterate over all clusters
        for clusterId in AdminConfig.list('ServerCluster').split(NEWLINE):
            dsIds = rd.readConfigDataUsingParentId(clusterId, 'cluster', 'DataSource')
            jndiNames = []
            dsMap = {}

            # Iterate over all data sources for the cluster
            for dataSource in dsIds:
                values = self.getBPMReader().loadDataSource(dataSource)
                jndiName = values['jndiName']
                jndiNames.append(jndiName)
                dsMap[jndiName] = values
            #endFor

            clusterType = self.decideCluster(jndiNames)
            if(clusterType != None):
                self.clusterDS[clusterType] = dsMap
                self.clusterName[clusterType] = AdminConfig.showAttribute(clusterId, 'name')
                self.LOGGER.log("CRWBP0110I", [clusterType, self.clusterName[clusterType]])
            #endIf
        #endFor

        self.collect("WBI_DEPLOYMENT_ENVIRONMENT_TOPOLOGY", "Custom")
        self.collect("WBI_ENVIRONMENT_TYPE", "custom")
        self.collect("WBI_APPLICATION_CLUSTER_NAME", self.clusterName['Application'])
        self.collect("WBI_SUPPORT_CLUSTER_NAME", self.clusterName['Support'])
        self.collect("WBI_MESSAGING_CLUSTER_NAME", self.clusterName['Messaging'])

        if self.clusterName.has_key('Web'):
            self.collect("WBI_WEB_CLUSTER_NAME", self.clusterName['Web'])
        else:
            self.collect('WBI_WEB_CLUSTER_NAME', '')

        for typeDSMap in self.clusterDS.items():
            (cType, dsList) = typeDSMap
            for jndiValues in dsList.items():
                (jndi, values) = jndiValues
                self.handleComponent(cType, values)
            #endFor
        #endFor

        self.handleJMSAuth("WBI_JMS_", "CommonEventInfrastructureJMSAuthAlias")
        if(self.cellType == "WPS"):
            self.handleJMSAuth("WBI_JMS_BPC_", "BPC_Auth_Alias")
    #endDef


    '''
    '''
    def calcPrefix(self, clusterType, dataSource):
        jndiName = dataSource['jndiName']
        if clusterType == 'Messaging':
            kind = self.messagingJNDI[jndiName]
            prefix = {
                      'BPC' : lambda: 'WPS_DB_BPC_ME_',
                      'CEI' : lambda: 'WBI_DB_CEI_ME_',
                      'SCAAPP' : lambda: 'WBI_DB_SCA_APP_ME_',
                      'SCASYS' : lambda: 'WBI_DB_SCA_SYS_ME_',
                     }[kind]()
            return prefix
        #endIf
        if (jndiName == 'jdbc/BPEDB'):
            if (clusterType == 'Application'):
                prefix = 'WPS_DB_BPC_'
            elif (clusterType == 'Web'):
                prefix = 'WPS_DB_BPC_WEB_'
            else:
                return ''
        #endIf
        else:
            if ((self.bpc.matcher(java.lang.String(jndiName)).matches() > 0) or 
                (self.cei.matcher(java.lang.String(jndiName)).matches() > 0) or
                (self.scaa.matcher(java.lang.String(jndiName)).matches() > 0) or
                (self.scas.matcher(java.lang.String(jndiName)).matches() > 0)):
                return ''
            else:
                prefix = {
                          'jdbc/cei' : lambda: 'WBI_DB_CEI_',
                          'jdbc/eventcatalog' : lambda: 'UNDEFINED',
                          'jdbc/bpm/BusinessSpace' : lambda: 'WBI_DB_BSPACE_',
                          }[dataSource['jndiName']]()
            #endIf
        #endIf
        return prefix
    #endDef


    '''
    Cell level data sources
        # jdbc/OBSVRDB_Support
        # jdbc/OBSVRDB_Support_WEB
    '''
    def handleObserver(self, ds, prefix):
        self.LOGGER.log("CRWBP0112I", prefix)
        authAliasId = ds['authDataAlias']
        authAlias = self.getBPMReader().findCellAuthAlias(authAliasId)

        userName = authAlias['userId']
        self.collect(prefix + "USER_NAME", userName)
        password = authAlias['password']
        self.collect(prefix + "PASSWORD", self.encrypt(prefix + "PASSWORD", password))

        # Schema settings
        schema = None

        if(self.isOracleDB() == 1):
            # Oracle needs to have user name == schema name
            schema = userName
        else:
            # For DB2 we can't deduce the schema (yet)
            schema = DR_DEFAULT_VALUE

        self.collect(prefix + "SCHEMA_NAME", schema)
    # endDef

    '''
    '''
    def handleComponent(self, clusterType, dataSource):
        dsName = dataSource['name']
        jndiName = dataSource['jndiName']
        self.LOGGER.log("CRWBP0111I", [dsName, clusterType])

        prefix = self.calcPrefix(clusterType, dataSource)
        if(prefix == "UNDEFINED"):
            return

        userName = ''
        if dataSource.has_key('authDataAlias'):
            authAliasId = dataSource['authDataAlias']
            authAlias = self.getBPMReader().findCellAuthAlias(authAliasId)
            if authAlias.has_key('userId'):
                userName = authAlias['userId']
                self.collect(prefix + "USER_NAME", userName)
            #endIf
            if authAlias.has_key('password'):
                password = authAlias['password']
                self.collect(prefix + "PASSWORD", self.encrypt(prefix + "PASSWORD", password))
            #endIf
        #endIf

        # Schema settings
        schema = None
        if(self.isOracleDB() == 1):
            # Oracle needs to have user name == schema name
            schema = userName
        else:
            # DB2 schema processing
            # This schema is not part of a standard WAS datasource
            if ((clusterType is 'Application') or (clusterType is 'Web')) and jndiName == 'jdbc/BPEDB':
                schema = self.getBPMReader().findBPMSchema(self.clusterName[clusterType], 'BPCDataStore', 'schemaName')
            # This schema is not part of a standard WAS datasource
            elif clusterType is 'Support' and jndiName == 'jdbc/bpm/BusinessSpace':
                schema = self.getBPMReader().findBPMSchema(self.clusterName[clusterType], 'BusinessSpaceServer', 'schemaName')
            else: #Messaging cluster
                for sibDatasource in AdminConfig.list('SIBDatastore').split(NEWLINE):
                    sibDSName = AdminConfig.showAttribute(sibDatasource, 'dataSourceName')
                    if (sibDSName == jndiName):
                        schema = AdminConfig.showAttribute(sibDatasource, 'schemaName')
                        break
                #endFor
            #endIf
        #endIf
        if (schema == None or schema == ''):
            schema = dataSource['currentSchema']
        self.collect(prefix + "SCHEMA_NAME", schema)

        if prefix == 'WPS_DB_BPC_':
            self.collect(prefix + "NAME", dataSource['databaseName'])
    #endDef

    def handleJMSAuth(self, prefix, alias):
        self.LOGGER.log("CRWBP0108I", prefix)
        authAlias = self.getBPMReader().findCellAuthAlias(alias)

        userName = authAlias['userId']
        self.collect(prefix + "USER_NAME", userName)

        password = authAlias['password']
        self.collect(prefix + "PASSWORD", self.encrypt(prefix + "PASSWORD", password))

        self.collect(prefix + "GROUP", "")
    #endDef

#endClass