'''
Created on Nov 19, 2010

@author: Per Bergman
'''

import java
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

'''
classdocs
'''
class OracleUtil:

    def __init__(self):
        self.LOGGER = _Logger("OracleUtil", MessageManager.RB_WEBSPHERE_WAS)
        self.oraURL = java.util.regex.Pattern.compile('jdbc:oracle:thin:@(.+):(\d+):(\w+)')
    #endDef

    def splitConnection(self, con):
        desc = con.find("(DESCRIPTION=")
        if(desc == -1):
            return self.splitJDBC(con)
        else:
            return self.splitTNS(con)

    '''
    "jdbc:oracle:thin:@rbf-sol10-ora.tivlab.austin.ibm.com:1521:orcl"
    '''
    def splitJDBC(self, jdbcURL):
        oraUrlMatcher = self.oraURL.matcher(java.lang.String(jdbcURL))
        if (oraUrlMatcher.matches() > 0):
            host = oraUrlMatcher.group(1)
            port = oraUrlMatcher.group(2)
            name = oraUrlMatcher.group(3)
            return name, [host, port]
    #endDef

    def parse(self, s, seek, pos):
        pos=s.find(seek, pos)
        pos=pos+1
        eqsign=s.find('=', pos)
        paren=s.find(')', pos)
        return s[eqsign+1:paren], paren+1
    #endDef

    '''
    Simple TNS:
    "jdbc:oracle:thin:@(DESCRIPTION=(ENABLE=BROKEN)(ADDRESS=(PROTOCOL=TCP)
      (HOST=rbf-sol10-ora.tivlab.austin.ibm.com)(PORT=1521))(CONNECT_DATA=(SERVICE_NAME=orcl)))"

     RAC TNS:
     "jdbc:oracle:thin:@(DESCRIPTION=(ENABLE=BROKEN)(ADDRESS=(PROTOCOL=TCP)(HOST=10.235.13.33)(PORT=1521))
      (ADDRESS=(PROTOCOL=TCP)(HOST=10.235.13.34)(PORT=1521))(CONNECT_DATA=(SERVICE_NAME=proc2)))"
      
     Standard JDBC URL:
     "jdbc:oracle:thin:@rbf-sol10-ora.tivlab.austin.ibm.com:1521:orcl"
    '''
    def splitTNS(self, s):
        ret=[]
        name=None
        if (s.find("(SERVICE_NAME=") != -1):
            ## Simple or RAC TNS url format
            name, tmp = self.parse(s, '(SERVICE_NAME=', 0)
            addr = s.find("(ADDRESS=")
            while(addr != -1):
                st,en=self.parse(s, '(HOST=', addr)
                ret.append(st)
                st,en=self.parse(s, '(PORT=', en)
                ret.append(st)
                addr = s.find("(ADDRESS=", en)
            #endWhile
        else:
            ## Standard jdbc url format
            tmp=(s.split('@')[1]).split(':')
            # name=DB_NAME
            name = tmp[2]
            # ret = ['DB_HOST', 'DB_PORT']
            ret.append(tmp[0])
            ret.append(tmp[1])
        #endIf
        return name, ret
    #endDef

#endClass
