-- The WSRR installer will replace the DBNAME, DBUSER and DB2TSDIR variables in this file

-- Licensed Materials - Property of IBM
-- 5724-N72 5655-R41
-- (c) Copyright IBM Corp. 2006, 2007 All Rights Reserved
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with
-- IBM Corp.

-- R2:  CREATE DATABASE @DBNAME@ ALIAS @DBNAME@ USING CODESET UTF-8 TERRITORY US COLLATE USING IDENTITY;
-- AUTOCONFIGURE:
-- ==============
-- DB2 will automatically configure 25% of physical memory for the database
-- and the USERSPACE1 tablespace will automatically be configured to use
-- automatic storage, with an initial size of 500M
CREATE DATABASE @DBNAME@ ON @DB2TSDIR@ ALIAS @DBNAME@ 
       USING CODESET UTF-8 TERRITORY US 
       COLLATE USING IDENTITY
       USER TABLESPACE
         MANAGED BY AUTOMATIC STORAGE
         AUTORESIZE YES
         INITIALSIZE 500 M
         INCREASESIZE 10 PERCENT
         MAXSIZE NONE;

UPDATE DBM CFG USING ASLHEAPSZ 256 IMMEDIATE;
UPDATE DBM CFG USING agent_stack_sz  256 IMMEDIATE;
UPDATE DBM CFG USING QUERY_HEAP_SZ 4096 IMMEDIATE;
UPDATE DBM CFG using DFT_MON_TIMESTAMP OFF IMMEDIATE; 

UPDATE DATABASE CONFIGURATION FOR @DBNAME@ USING STMTHEAP        32768;
UPDATE DATABASE CONFIGURATION FOR @DBNAME@ USING APPLHEAPSZ      4096 IMMEDIATE;
UPDATE DATABASE CONFIGURATION FOR @DBNAME@ USING AVG_APPLS       5    IMMEDIATE;
UPDATE DATABASE CONFIGURATION FOR @DBNAME@ USING DBHEAP          4096 IMMEDIATE ;
UPDATE DATABASE CONFIGURATION FOR @DBNAME@ USING SHEAPTHRES_SHR  AUTOMATIC;
UPDATE DATABASE CONFIGURATION FOR @DBNAME@ USING LOGBUFSZ        2048 IMMEDIATE ;
UPDATE DATABASE CONFIGURATION FOR @DBNAME@ USING LOGFILSIZ       1000;
UPDATE DATABASE CONFIGURATION FOR @DBNAME@ USING LOGPRIMARY      50;
UPDATE DATABASE CONFIGURATION FOR @DBNAME@ USING LOGSECOND       200;

CONNECT TO @DBNAME@;

GRANT DBADM,CREATETAB,BINDADD,CONNECT,CREATE_NOT_FENCED_ROUTINE,IMPLICIT_SCHEMA,LOAD,CREATE_EXTERNAL_ROUTINE,QUIESCE_CONNECT ON DATABASE  TO USER @DBUSER@;

CREATE BUFFERPOOL @DBNAME@ IMMEDIATE 
    SIZE 1000 AUTOMATIC 
    NUMBLOCKPAGES 256 
    BLOCKSIZE 32
    PAGESIZE 32K ;

CREATE BUFFERPOOL @DBNAME@TMP IMMEDIATE 
    SIZE 1000 AUTOMATIC 
    NUMBLOCKPAGES 256
    BLOCKSIZE 32
    PAGESIZE 32K; 
CREATE BUFFERPOOL @DBNAME@TS IMMEDIATE 
    SIZE 1000 AUTOMATIC 
    NUMBLOCKPAGES 256
    BLOCKSIZE 32
    PAGESIZE 32K; 
-- Need this as the IBMDEFAULTBP is the wrong page size..
CREATE BUFFERPOOL @DBNAME@SCHED IMMEDIATE 
    SIZE 500 AUTOMATIC 
    NUMBLOCKPAGES 64
    BLOCKSIZE 32
    PAGESIZE 32 K; 

-- Create a DMS tablespace with 32k pagesize to hold wide tables
-- whose storage is automatically managed by DB2
CREATE REGULAR TABLESPACE @DBNAME@DATA
       IN DATABASE PARTITION GROUP IBMDEFAULTGROUP 
       PAGESIZE 32768 
       MANAGED BY AUTOMATIC STORAGE
       AUTORESIZE YES
       INITIALSIZE 200 M
       INCREASESIZE 10 PERCENT
       MAXSIZE NONE
       EXTENTSIZE 16
       BUFFERPOOL @DBNAME@
       FILE SYSTEM CACHING  
       DROPPED TABLE RECOVERY ON;

CREATE REGULAR TABLESPACE @DBNAME@TS
       IN DATABASE PARTITION GROUP IBMDEFAULTGROUP 
       PAGESIZE 32768 
       MANAGED BY AUTOMATIC STORAGE
       AUTORESIZE YES
       INITIALSIZE 100 M
       INCREASESIZE 10 PERCENT
       MAXSIZE NONE
       EXTENTSIZE 16
       BUFFERPOOL @DBNAME@TS
       FILE SYSTEM CACHING  
       DROPPED TABLE RECOVERY ON;

CREATE REGULAR TABLESPACE @DBNAME@SCHED
       IN DATABASE PARTITION GROUP IBMDEFAULTGROUP 
       PAGESIZE 32768 
       MANAGED BY AUTOMATIC STORAGE
       AUTORESIZE YES
       INITIALSIZE 50 M
       INCREASESIZE 10 PERCENT
       MAXSIZE NONE
       EXTENTSIZE 16
       BUFFERPOOL @DBNAME@SCHED
       FILE SYSTEM CACHING  
       DROPPED TABLE RECOVERY ON;

-- Create a temorary SMS tablespace for the XMETA regular tablespace
-- whose storage is automatically managed by DB2
CREATE TEMPORARY TABLESPACE @DBNAME@TEMP 
       IN DATABASE PARTITION GROUP IBMTEMPGROUP 
       PAGESIZE 32768 
       MANAGED BY AUTOMATIC STORAGE
       EXTENTSIZE 16
       BUFFERPOOL @DBNAME@TMP
       FILE SYSTEM CACHING ;

COMMENT ON TABLESPACE @DBNAME@DATA IS '32 page WSRR Data space';
COMMENT ON TABLESPACE @DBNAME@TS IS '32 page WSRR TS Data space';
COMMENT ON TABLESPACE @DBNAME@SCHED IS '32 page WSRR Scheduler Data space';
COMMENT ON TABLESPACE @DBNAME@TEMP IS '32 page swap';

COMMIT WORK;
CONNECT RESET;
TERMINATE; 
